/**********************************************************
 * libmp3splt -- library based on mp3splt,
 *               for mp3/ogg splitting without decoding
 *
 * Copyright (c) 2002-2005 M. Trotta - <matteo.trotta@lib.unimib.it>
 * Copyright (c) 2005-2006 Munteanu Alexandru - io_alex_2002@yahoo.fr
 *
 *********************************************************/

/**********************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307,
 * USA.
 *********************************************************/

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#ifndef NO_ID3TAG
#include <id3tag.h>
#endif

#include "mp3splt_utils.h"

#if !HAVE_FSEEKO
#define fseeko fseek
#define ftello ftell
#endif

//layer, bitrate..
const int tabsel_123[2][3][16] = {
  { {128,32,64,96,128,160,192,224,256,288,320,352,384,416,448,},
    {128,32,48,56, 64, 80, 96,112,128,160,192,224,256,320,384,},
    {128,32,40,48, 56, 64, 80, 96,112,128,160,192,224,256,320,} },
  
  { {128,32,48,56,64,80,96,112,128,144,160,176,192,224,256,},
    {128,8,16,24,32,40,48,56,64,80,96,112,128,144,160,},
    {128,8,16,24,32,40,48,56,64,80,96,112,128,144,160,} }
};

//categories of mp3 songs
const char id3_categories[MP3SPLT_GENRENUM][25] = {
  {"Blues"}, {"Classic Rock"}, {"Country"}, {"Dance"}, 
  {"Disco"},{"Funk"},{"Grunge"},{"Hip-Hop"},{"Jazz"},
  {"Metal"},{"New Age"},{"Oldies"}, {"Other"}, {"Pop"},
  {"R&B"}, {"Rap"}, {"Reggae"}, {"Rock"}, {"Techno"},
  {"Industrial"}, {"Alternative"}, {"Ska"}, {"Death metal"},
  {"Pranks"}, {"Soundtrack"}, {"Euro-Techno"},
  {"Ambient"}, {"Trip-hop"}, {"Vocal"}, {"Jazz+Funk"},
  {"Fusion"}, {"Trance"}, {"Classical"}, {"Instrumental"},
  {"Acid"}, {"House"}, {"Game"}, {"Sound clip"}, {"Gospel"},
  {"Noise"}, {"Alt. Rock"}, {"Bass"}, {"Soul"}, {"Punk"}, 
  {"Space"}, {"Meditative"}, {"Instrumental pop"}, 
  {"Instrumental rock"}, {"Ethnic"}, {"Gothic"},{"Darkwave"},
  {"Techno-Industrial"},{"Electronic"},{"Pop-Folk"},{"Eurodance"},
  {"Dream"},{"Southern Rock"},{"Comedy"}, {"Cult"},{"Gangsta"},
  {"Top 40"},{"Christian Rap"},{"Pop/Funk"}, {"Jungle"},
  {"Native American"},{"Cabaret"},{"New Wave"}, {"Psychedelic"},
  {"Rave"},{"Showtunes"},{"Trailer"}, {"Lo-Fi"},{"Tribal"},
  {"Acid Punk"},{"Acid Jazz"}, {"Polka"}, {"Retro"},
  {"Musical"},{"Rock & Roll"},{"Hard Rock"}, {"misc"}, {"misc"},
};

const char unsigned id3genre[MP3SPLT_GENRENUM] = 
  { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 
    0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F,
    0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F,
    0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F,
    0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F,
    0xFF };

#ifndef NO_ID3TAG
//puts a original field on id3 conforming to frame_type
void put_original_id3_frame(splt_state *state,
			    struct id3_tag *id3tag,
			    char *frame_type,
			    int id_type)
{
  struct id3_frame *frame;
  id3_ucs4_t *ucs4;
  id3_latin1_t *latin1;
  
  //we get the artist
  frame = id3_tag_findframe(id3tag, frame_type,0);
  if (frame != NULL)
    {
      if (id_type == MP3SPLT_ID3_COMMENT)
	{
	  ucs4 = (id3_ucs4_t *)
	    id3_field_getfullstring(&frame->fields[3]);
	}
      else
	{
	  ucs4 = (id3_ucs4_t *) 
	    id3_field_getstrings(&frame->fields[1],0);
	}
      if (ucs4 != NULL)
	{
	  latin1 = 
	    id3_ucs4_latin1duplicate (ucs4);
	  if (latin1 != NULL)
	    {
	      switch (id_type)
		{
		case MP3SPLT_ID3_ALBUM:
		  snprintf(state->original_tags.album,
			   31,"%s",(char *)latin1);
		  break;
		case MP3SPLT_ID3_ARTIST:
		  snprintf(state->original_tags.artist,
			   31,"%s",(char *)latin1);
		  break;
		case MP3SPLT_ID3_TITLE:
		  if (strcmp(frame_type,ID3_FRAME_TITLE) == 0)
		    snprintf(state->original_tags.title,
			     31,"%s",(char *)latin1);
		  break;
		case MP3SPLT_ID3_YEAR:
		  snprintf(state->original_tags.year,
			   5,"%s",(char *)latin1);
		  break;
		case MP3SPLT_ID3_TRACK:
		  state->original_tags.track = 
		    atof((char*)latin1);
		  break;
		case MP3SPLT_ID3_COMMENT:
		  snprintf(state->original_tags.comment,
			   31,"%s",(char *)latin1);
		  break;
		case MP3SPLT_ID3_GENRE:
		  state->original_tags.genre = 
		    getgenre((char *)latin1);
		  
		  int number = 80;
		  number = atoi((char *)latin1);
		  //if we have a number returned by latin1
		  if ((number != 0) &&
		      (state->original_tags.genre == 0xFF))
		    {
		      state->original_tags.genre = number;
		    }
		  //if we have 0 returned
		  if (strcmp((char*)latin1, "0") == 0)
		    {
		      //12 mean Otherx
		      state->original_tags.genre = 12;
		    }
		  break;
		default:
		  break;
		}
	    }
	  free(latin1);
	}
    }
}

//this function puts the original id3 tags if we had libid3tag enabled
//at compilation time
void put_original_id3_tags(char *filename,splt_state *state,
			   int *tag_error)
{
  //we get the id3 from the original file
  //using libid3tag
  struct id3_file *id3file;
  struct id3_tag *id3tag;
  
  //we open the file
  id3file = id3_file_open(filename,
			  ID3_FILE_MODE_READONLY);
  
  if (id3file == NULL)
    {
      *tag_error = MP3SPLT_ERROR_CANNOT_OPEN_FILE;
    }
  else
    {
      //we get the real tags
      id3tag = id3_file_tag (id3file);
      
      //we copy original tag to our new tag
      //we get the artist
      put_original_id3_frame(state,id3tag,ID3_FRAME_ARTIST,
			     MP3SPLT_ID3_ARTIST);
      put_original_id3_frame(state,id3tag,ID3_FRAME_ALBUM,
			     MP3SPLT_ID3_ALBUM);
      put_original_id3_frame(state,id3tag,ID3_FRAME_TITLE,
			     MP3SPLT_ID3_TITLE);
      put_original_id3_frame(state,id3tag,ID3_FRAME_YEAR,
			     MP3SPLT_ID3_YEAR);
      put_original_id3_frame(state,id3tag,ID3_FRAME_GENRE,
			     MP3SPLT_ID3_GENRE);
      put_original_id3_frame(state,id3tag,ID3_FRAME_COMMENT,
			     MP3SPLT_ID3_COMMENT);
      put_original_id3_frame(state,id3tag,ID3_FRAME_TRACK,
			     MP3SPLT_ID3_TRACK);
      
      //we close the file
      id3_file_close(id3file);
    }
}
#endif

//return id3 buffer as string (the char *id is filled)
char *id3 (char *id, char *title, char *artist, 
	   char *album, char *year, unsigned char genre, 
	   char *comment, short track)
{
  char buffer[30];
  int j=3,i;
  strncpy(id, MP3SPLT_TAG, 3);

  memset(buffer, '\0', 30);
  if (title!=NULL) strncpy(buffer, title, 30);
  for (i=0; i<30; i++) id[j++]=buffer[i];
  
  memset(buffer, '\0', 30);
  if (artist!=NULL) strncpy(buffer, artist, 30);
  for (i=0; i<30; i++) id[j++]=buffer[i];

  memset(buffer, '\0', 30);
  if (album!=NULL) strncpy(buffer, album, 30);
  for (i=0; i<30; i++) id[j++]=buffer[i];

  memset(buffer, '\0', 30);
  if (year!=NULL) strncpy(buffer, year, 4);
  for (i=0; i<4; i++) id[j++]=buffer[i];
  
  memset(buffer, '\0', 30);
  if (comment!=NULL) strncpy(buffer, comment, 30);
  for (i=0; i<30; i++) 
    id[j++]=buffer[i];
  //if we have a positive track
  if (track != -1)
    {
      if (track!=0x00)
	id[j-1] = (char) track;
    }  
  id[j]=(char) genre;
  
  return id;
}

//returns the genre of the song, mp3splt used this in cddb search
unsigned char getgenre (char *s)
{
  int i;
  for (i=0; i<MP3SPLT_GENRENUM; i++) {
    if (strncmp(s, id3_categories[i], strlen(s))==0)
      return id3genre[i];
  }
  return 0xFF;
}

//does nothing important for libmp3splt
//review this..
void checksync (mp3_state *state)
{
  //char junk[32];
  //fprintf(stderr, "\nWarning: Too many sync errors! This may not be a mp3 file. Continue? (y/n) ");
  //fgets(junk, 31, stdin);
  //if (junk[0]=='y')
  
  //we don't use user interactivity in a library
  //always continue
  state->syncdetect = 0;
  
  //else error("Aborted.",125);
}

//calculates bitrate
int c_bitrate (unsigned long head)
{
  if ((head & 0xffe00000) != 0xffe00000) return 0;
  if (!((head>>17)&3)) return 0;
  if (((head>>12)&0xf) == 0xf) return 0;
  if (!((head >> 12) & 0xf)) return 0;
  if (((head>>10)&0x3) == 0x3 ) return 0;
  if (((head >> 19) & 1)==1 && ((head>>17)&3)==3 && ((head>>16)&1)==1) return 0;
  if ((head & 0xffff0000) == 0xfffe0000) return 0;
  
  return ((head>>12)&0xf);
}

//make mp3 header bitrate, padding, offset, framesize
struct header makehead (unsigned long headword, struct mp3 mp3f, struct header head, off_t ptr)
{
  head.ptr = ptr;
  head.bitrate = tabsel_123[1 - mp3f.mpgid][mp3f.layer-1][c_bitrate(headword)];
  head.padding = ((headword>>9)&0x1);
  head.framesize = (head.bitrate*144000)/(mp3f.freq<<(1 - mp3f.mpgid)) + head.padding;
  return head;
}

//finds first header from start_pos. Returns -1 if no header is found
off_t findhead (mp3_state *state, off_t start)
{
  if (getword(state->file_input, start, SEEK_SET, &state->headw) == -1)
    {
      return -1;
    }
  if (feof(state->file_input)) 
    {
      return -1;
    }
  while (!(c_bitrate(state->headw))) {
    if (feof(state->file_input)) 
      {
	return -1;
      }
    state->headw <<= 8;
    state->headw |= fgetc(state->file_input);
    start++;
  }
  return start;
}

// Finds first valid header from start. Will work with high probabilty, i hope :)
off_t findvalidhead (mp3_state *state, off_t start)
{
  off_t begin;
  struct header h;
  
  begin = findhead(state, start);
  do {
    start = begin;
    if (start == -1) 
      break;
    h = makehead (state->headw, state->mp3file, h, start);
    begin = findhead(state, (start + 1));
  } while (begin!=(start + h.framesize));
  
  return start;
}

//search for ID3 v1 tag, "TAG" sequence
//if found returns offset of mp3 data
int getid3v1 (FILE *file_input)
{
  if (fseeko(file_input, (off_t) -128, SEEK_END)==-1)
    return 0;
  if (fgetc(file_input)=='T')
    if (fgetc(file_input)=='A')
      if (fgetc(file_input)=='G')
	return -128;
  return 0;
}

//check if there is a ID3v2. 
//if found, it returns offset of mp3 data.
off_t getid3v2 (FILE *in, off_t start)
{
  unsigned long oword = 0;
  if (fseeko(in, start, SEEK_SET)==-1)
    return 0;
  if (fgetc(in)=='I')
    if (fgetc(in)=='D')
      if (fgetc(in)=='3') {
	int i;
	if (fseeko(in, (off_t) 3, SEEK_CUR)==-1)
	  return 0;
	for (i=0; i<4; i++)
	  oword = (oword << 7) | fgetc(in);
	return (off_t) (oword);
      }

  return 0;
}

//finds xing info offset and returns it?
int xing_info_off(mp3_state *state)
{
  unsigned long headw = 0;
  int i;

  for (i=0; i<state->mp3file.xing; i++)
    {
      if ((headw==MP3SPLT_XING_MAGIC) || 
	  (headw==MP3SPLT_INFO_MAGIC)) // "Xing" or "Info"
	return i;
      headw <<= 8;
      headw |= state->mp3file.xingbuffer[i];
    }
  
  return 0;
}

//get a frame
int get_frame(mp3_state *state)
{
  if(state->stream.buffer==NULL || 
     state->stream.error==MAD_ERROR_BUFLEN)
    {
      size_t readSize, remaining;
      unsigned char *readStart;

      if (feof(state->file_input))
	{
	  return -2;
	}
      
      if(state->stream.next_frame!=NULL)
	{
	  remaining = state->stream.bufend - state->stream.next_frame;
	  memmove(state->inputBuffer, state->stream.next_frame, remaining);
	  readStart = state->inputBuffer + remaining;
	  readSize = MP3SPLT_MAD_BSIZE - remaining;
	}
      else
	{
	  readSize = MP3SPLT_MAD_BSIZE;
	  readStart=state->inputBuffer;
	  remaining=0;
	}
      readSize=fread(readStart, 1, readSize, state->file_input);
      if (readSize <= 0)
	return -2;
      state->buf_len = readSize + remaining;
      state->bytes += readSize;
      mad_stream_buffer(&state->stream, state->inputBuffer, 
			readSize+remaining);
      state->stream.error = MAD_ERROR_NONE;
    }
  
  return mad_frame_decode(&state->frame,&state->stream);
}

//used by mp3split and mp3_scan_silence
//gets a frame and checks for its validity
//returns 1 if ok, -1 if end of file, 0 if nothing ?
//sets the state->data_ptr the pointer to the frame
//and the state->data_len the length of the frame
int get_valid_frame(mp3_state *state)
{
  int ok = 0;
  do
    {
      int ret = get_frame(state);
      if(ret)
	{
	  if (ret == -2)
	    {
	      return -1;
	    }
	  if (state->stream.error == MAD_ERROR_LOSTSYNC)
	    {
	      state->syncerrors++;
	      if ((state->syncdetect)&&
		  (state->syncerrors>MP3SPLT_MAXSYNC))
		{
		  checksync(state);
		}
	    }
	  if(MAD_RECOVERABLE(state->stream.error))
	    {
	      continue;
	    }
	  else
	    if(state->stream.error==MAD_ERROR_BUFLEN)
	      {
		continue;
	      }
	    else
	      {
		break;
	      }
	}
      else
	{
	  //the important stuff
	  state->data_ptr = (unsigned char *) state->stream.this_frame;
	  if(state->stream.next_frame!=NULL)
	    {
	      state->data_len = (long) (state->stream.next_frame - state->stream.this_frame);
	    }
	  ok = 1;
	}
      
    } while (!ok);
  
  return ok;
}

//puts in the state informations about mp3 file
//i think it also reads the file
//must be called before mp3split()
//enables framemode if xing header found 
//xing header is often associated with VBR (variable bit rate)
mp3_state *mp3info(FILE *file_input, splt_state *sp_state,
		   int framemode, int *error)
{
  mp3_state *state = sp_state->mstate;
  
  int prev = -1, len;
  
  if ((state = malloc (sizeof(mp3_state)))==NULL)
    {
      *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
      return NULL;
    }
  
  memset(state, 0x0, sizeof(mp3_state));
  
  //always quiet
  state->syncdetect = 0;
  
  //we initialise default values
  state->frames = 1;
  state->end = 0;
  state->first = 1;
  state->syncerrors = 0;
  state->file_input = file_input;
  state->framemode = framemode;
  state->headw = 0;
  state->mp3file.xing = 0;
  state->mp3file.xing_offset = 0;
  state->mp3file.xingbuffer = NULL;
  state->mp3file.len = flength(file_input);
  sp_state->split.total_time = 0;
  state->data_ptr = NULL;
  state->data_len = 0;
  state->buf_len = 0;
  state->bytes = 0;
  
  state->silence_list = NULL;
  
  //we initialise the mad structures
  mad_stream_init(&state->stream);
  mad_frame_init(&state->frame);
  mad_synth_init(&state->synth);
  mad_timer_reset(&state->timer);
  
  //we read mp3 infos and set pointers to read the mp3 data
  do
    {
      int ret = get_frame(state);
      
      if (ret==-2)
	{
	  //we free memory allocated by mad_frame_decode(..)
	  mad_stream_finish(&state->stream);
	  mad_frame_finish(&state->frame);
	  *error = MP3SPLT_ERROR_INVALID_MP3;
	  return NULL;
	}
      
      //we end the cycle if
      if ((prev == 0) && 
	  ((ret == 0) || 
	   (state->stream.error==MAD_ERROR_BUFLEN)))
	{
	  break;
	}
      
      //if we have succeeded to read a frame
      if (ret == 0)
	{
	  //we set pointer to the frame
	  state->data_ptr = (unsigned char *) state->stream.this_frame;
	  //we set length of frame
	  if(state->stream.next_frame!=NULL)
	    state->data_len = (long) (state->stream.next_frame - state->stream.this_frame);
	  
	  if (state->stream.anc_bitlen > 64)
	    {
	      int tag = 0;
	      struct mad_bitptr ptr = state->stream.anc_ptr;
	      struct mad_bitptr start = ptr;
	      //we search for xing (variable bit rate)
	      unsigned long xing_word = mad_bit_read(&ptr, 32);
	      if ((xing_word==MP3SPLT_XING_MAGIC) ||
		  (xing_word==MP3SPLT_INFO_MAGIC))
		tag = 1;
	      //Handle misplaced Xing header in mp3 files with CRC
	      else if (xing_word == ((MP3SPLT_XING_MAGIC << 16) & 0xffffffffL) 
		       || xing_word == ((MP3SPLT_INFO_MAGIC << 16) & 0xffffffffL))
		{
		  ptr = start;
		  mad_bit_skip(&ptr, 16);
		  tag = 1;
		}
	      
	      //if we have xing, put infos
	      if (tag)
		{
		  xing_word = mad_bit_read(&ptr, 32);
		  if (xing_word & MP3SPLT_XING_FRAMES)
		    {
		      mad_timer_t total;
		      state->frames = mad_bit_read(&ptr, 32);
		      total = state->frame.header.duration;
		      mad_timer_multiply(&total, state->frames);
		      sp_state->split.total_time = 
			mad_timer_count(total, MAD_UNITS_CENTISECONDS);
		    }
		  
		  if (xing_word & MP3SPLT_XING_BYTES) {
		    if (state->mp3file.len == 0)
		      state->mp3file.len = mad_bit_read(&ptr, 32);
		  }
		  state->mp3file.xing = state->data_len;
		  if ((state->mp3file.xingbuffer = 
		       malloc(state->mp3file.xing))==NULL)
		    {
		      *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
		      return NULL;
		    }
		  memcpy(state->mp3file.xingbuffer, state->data_ptr, state->mp3file.xing);
		  state->mp3file.xing_offset = xing_info_off(state);
		  //set framemode true (because VBR)
		  sp_state->options.option_mp3_frame_mode = 1;
		  state->framemode = 1;
		  if (sp_state->split.put_message != NULL)
		    {
		      sp_state->split.put_message(MP3SPLT_FRAME_MODE_ENABLED);
		    }
		  continue;
		}
	    }
	}
      
      prev = ret;
      
    } while (1);
  
  len = (long) (state->buf_len - (state->data_ptr - state->inputBuffer));
  
  if (len < 0)
    {
      *error = MP3SPLT_ERROR_INVALID_MP3;
      //we free memory allocated by mad_frame_decode(..)
      mad_stream_finish(&state->stream);
      mad_frame_finish(&state->frame);
      return NULL;
    }
  
  //we put useful infos in the state
  state->mp3file.firsth = (off_t) (state->bytes - len);
  state->bytes = state->mp3file.firsth;
  state->headw = 
    (unsigned long) ((state->data_ptr[0] << 24) | 
		     (state->data_ptr[1] << 16) |
		     (state->data_ptr[2] << 8) | (state->data_ptr[3]));
  state->mp3file.mpgid = (int) ((state->headw >> 19)&1);
  state->mp3file.layer = state->frame.header.layer;
  
  state->mp3file.freq = state->frame.header.samplerate;
  state->mp3file.bitrate = state->frame.header.bitrate/MP3SPLT_BYTE;
  
  state->mp3file.firsthead = makehead(state->headw, state->mp3file, state->mp3file.firsthead, state->mp3file.firsth);
  /*fprintf(stderr,"mpgid = %d\n",
    state->mp3file.mpgid);*/
  state->mp3file.fps = (float) (state->mp3file.freq*(2-state->mp3file.mpgid));
  state->mp3file.fps /= MP3SPLT_PCM;
  
  //we put the channels stuff (mono, stereo)
  switch(state->frame.header.mode)
    {
    case MAD_MODE_SINGLE_CHANNEL:
      state->mp3file.channels = 0;
      break;
    case MAD_MODE_DUAL_CHANNEL:
      state->mp3file.channels = 1;
      break;
    case MAD_MODE_JOINT_STEREO:
      state->mp3file.channels = 2;
      break;
    case MAD_MODE_STEREO:
      state->mp3file.channels = 3;
      break;
    default:
      state->mp3file.channels = 4;
      break;
    }
  
  //we put the total time for constant bit rate
  //if it was not set for the variable bit rate
  if (sp_state->split.total_time == 0)
    {
      if (state->mp3file.len > 0)
	sp_state->split.total_time = (long)
	  ((state->mp3file.len - state->mp3file.firsth) 
	   / state->mp3file.bitrate) * 100;
    }
  
  //we free memory allocated by mad_frame_decode(..)
  mad_stream_finish(&state->stream);
  mad_frame_finish(&state->frame);
  mad_synth_finish(&state->synth);
  
  return state;
}

//put the song tags
char *put_mp3_song_tags(char *id3buffer,char *filename,
			splt_state *state, int *error)
{
  char *id3_data = NULL;
  
  //if the file is mp3
  //if we have the original_tags option, 
  //we put the original tags
  clean_original_id3(state);
  if (state->options.tags == MP3SPLT_TAGS_ORIGINAL_FILE)
    {
#ifndef NO_ID3TAG
      int tag_error = 0;
      put_original_id3_tags(filename,
			    state, &tag_error);
      if (tag_error < 0)
	{
	  *error = tag_error;
	}
      if ((state->original_tags.title[0] != '\0')
	  &&(state->original_tags.artist[0] != '\0'))
	{
	  //we put the id3 comments on id3_infos
	  id3_data = id3(id3buffer, 
			 state->original_tags.title,
			 state->original_tags.artist,
			 state->original_tags.album,
			 state->original_tags.year,
			 state->original_tags.genre, 
			 state->original_tags.comment,
			 state->original_tags.track);
	}
#endif
    }
  else
    {
      if (state->options.tags == MP3SPLT_CURRENT_TAGS)
	{
	  //only if we have the artist or the title
	  if ((state->tags.id[state->split.current_split].title[0] != '\0')
	      ||(state->tags.id[state->split.current_split].artist[0] != '\0'))
	    {
	      //we put the id3 comments on id3_infos
	      id3_data = id3(id3buffer, 
			     state->tags.id[state->split.current_split].title,
			     state->tags.id[state->split.current_split].artist,
			     state->tags.id[state->split.current_split].album,
			     state->tags.id[state->split.current_split].year,
			     state->tags.id[state->split.current_split].genre,
			     state->tags.id[state->split.current_split].comment,
			     state->split.current_split+1);
	    }
	}
    }
  
  return id3_data;
}

//the main mp3 split function
//filename is our filename
//state is our state
//id3 is the id3 buffer
//fbegin_sec is the beggining splitpoint
//fend_sec is the end splitpoint
//adjustoption is if we adjust with silence detection or not
//seekable is if we split in seekable mode or not
//threshold - see manual
////must be called after mp3info()
//returns possible error in error
void mp3split (char *filename, splt_state *sp_state, char *id3,
	       double fbegin_sec, double fend_sec, int *error)
{
  //we do the next split
  sp_state->split.current_split++;
  
  mp3_state *state = sp_state->mstate;
  
  /*fprintf(stderr, "split begin = %f\n",
	  fbegin_sec);
	  fprintf(stderr, "split end = %f\n",
	  fend_sec);*/

  /*  //crc check
  off_t begin,end;
  unsigned long crc=0;
  unsigned long fcrc=0;
  getword (sp_state->mstate->file_input, 0, SEEK_CUR, &fcrc);
  begin = ftello(sp_state->mstate->file_input);
  if (fseeko(sp_state->mstate->file_input, getid3v1(sp_state->mstate->file_input), SEEK_END)==-1)
    {
      //-1 means MP3SPLT_DEWRAP_ERR_FILE_NOT_WRAPED_DAMAGED
      fprintf(stderr, "file damaged\n");
    }
  end = ftello(sp_state->mstate->file_input);
  fprintf (stderr, "Check for file integrity: calculating CRC please wait... ");
  crc = c_crc (sp_state->mstate->file_input, begin, end);
  fprintf(stderr, "crc = %lu\n",crc);
  fprintf(stderr, "fcrc = %lu\n",fcrc);
  if (crc != fcrc)
    {
      fprintf (stderr, "BAD\n");
      }*/
  
  short adjustoption = sp_state->options.parameter_gap;
  short seekable = !sp_state->options.option_input_not_seekable;
  float threshold = sp_state->options.parameter_threshold;
  
  short status = -1, eof=0, check_bitrate=0;
  
  FILE *file_output = NULL;
  short writing = 0, finished=0;
  unsigned long fbegin=0;
  off_t wrote = 0;
  int len = 0;
  //for the progress
  unsigned long stopped_frames = 0;
  int progress_adjust_val = 2;
  if (adjustoption) 
    {
      progress_adjust_val = 4;
    }
  
  put_progress_text(sp_state,MP3SPLT_PROGRESS_CREATE);
  
  mad_stream_init(&state->stream);
  mad_frame_init(&state->frame);
  
  //if not seekable
  if (!seekable)
    {
      //for the stdout output, hmm..
      if (strcmp(filename, "-")==0)
	{
	  file_output = stdout;
	}
      else
	{
	  if (!(file_output=fopen(filename, "wb+")))
	    {
	      *error = MP3SPLT_ERROR_CANNOT_OPEN_DEST_FILE;
	      mad_frame_finish(&state->frame);
	      mad_stream_finish(&state->stream);
	      return;
	    }
	}
      
      //if we have the framemode
      if (state->framemode)
	{
	  long begin_c, end_c, time;
	  //convert seconds to hundreths
	  begin_c = (long) (fbegin_sec * 100);
	  if (fend_sec > 0)
	    {
	      end_c = (long) (fend_sec * 100);
	    }
	  else 
	    {
	      end_c = 0;
	    }
	  time = 0;
	  
	  do
	    {
	      //we write xing if vbr
	      if (!writing && (time >= begin_c))
		{
		  writing = 1;
		  fbegin = state->frames;
		  
		  if (state->mp3file.xing > 0)
		    {
		      wrote = fwrite(state->mp3file.xingbuffer, 1, state->mp3file.xing, file_output);
		      if (wrote <= 0)
			{
			  *error = MP3SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
			  mad_frame_finish(&state->frame);
			  mad_stream_finish(&state->stream);
			  return;
			}
		    }
		}
	      
	      //we do the split
	      if (writing)
		{
		  if (state->data_len > 0)
		    {
		      len = fwrite(state->data_ptr, 1, state->data_len, file_output);
		      if (len <= 0)
			{
			  *error = MP3SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
			  mad_frame_finish(&state->frame);
			  mad_stream_finish(&state->stream);
			  return;
			}
		      wrote = (off_t) (wrote + len);
		      state->data_len = 0;
		    }
		  if ((end_c > 0) && (time > end_c))
		    {
		      finished = 1;
		    }
		  if (eof || finished)
		    {
		      finished = 1;
		      status = 0;
		      break;
		    }
		}
	      
	      //if we have a progress callback function
	      if (sp_state->split.p_bar.progress != NULL)
		{
		  if (sp_state->options.split_mode == 
		      MP3SPLT_OPTION_TIME_MODE)
		    {
		      update_progress(sp_state,(float)(time-begin_c),
				      (float)(end_c-begin_c),1,0,
				      MP3SPLT_DEFAULT_PROGRESS_RATE);
		    }
		  else
		    {
		      update_progress(sp_state,(float)(time),
				      (float)(end_c),1,0,
				      MP3SPLT_DEFAULT_PROGRESS_RATE);
		    }
		}
	      
	      //we get next frame
	      switch (get_valid_frame(state))
		{
		case 1:
		  mad_timer_add(&state->timer, state->frame.header.duration);
		  state->frames++;
		  time = mad_timer_count(state->timer, MAD_UNITS_CENTISECONDS);
		  break;
		case 0:
		  break;
		case -1:
		  eof = 1;
		  break;
		}
	    } while (!finished);
	}
      //if we don't have the framemode
      else
	{
	  off_t begin = 0, end = 0;
	  if (fend_sec != -1)
	    {
	      end = (off_t) (fend_sec * state->mp3file.bitrate + state->mp3file.firsth);
	    }
	  
	  if (state->end == 0)
	    {
	      begin = (off_t) (fbegin_sec * state->mp3file.bitrate + state->mp3file.firsth);
	      
	      if ((state->bytes == begin) && (state->data_len > 0))
		{
		  len = (long) (state->inputBuffer + state->buf_len - state->data_ptr);
		  if (len < 0)
		    {
		      *error = MP3SPLT_ERROR_WHILE_READING_FILE;
		      mad_frame_finish(&state->frame);
		      mad_stream_finish(&state->stream);
		      return;
		    }
		  if (fwrite(state->data_ptr, 1, len, file_output) < 0)
		    {
		      *error = MP3SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
		      mad_frame_finish(&state->frame);
		      mad_stream_finish(&state->stream);
		      return;
		    }
		  wrote = (off_t) (wrote + len);
		  state->data_len = 0;
		}
	      else
		{
		  while (state->bytes < begin)
		    {
		      off_t to_read;
		      if (feof(state->file_input))
			{
			  *error = MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
			  mad_frame_finish(&state->frame);
			  mad_stream_finish(&state->stream);
			  return;
			}
		      to_read = (begin - state->bytes);
		      if (to_read > MP3SPLT_MAD_BSIZE)
			to_read = MP3SPLT_MAD_BSIZE;
		      if ((state->data_len = fread(state->inputBuffer, 1, to_read, state->file_input))<=0)
			{
			  *error = MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
			  mad_frame_finish(&state->frame);
			  mad_stream_finish(&state->stream);
			  return;
			}
		      state->bytes+=state->data_len;
		    }
		  
		  //we get next frame
		  switch (get_valid_frame(state))
		    {
		    case 1:
		      len = (long) (state->inputBuffer + state->buf_len - state->data_ptr);
		      if (len < 0)
			{
			  *error = MP3SPLT_ERROR_WHILE_READING_FILE;
			  mad_frame_finish(&state->frame);
			  mad_stream_finish(&state->stream);
			  return;
			}
		      if (fwrite(state->data_ptr, 1, len, file_output) < 0)
			{
			  *error = MP3SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
			  mad_frame_finish(&state->frame);
			  mad_stream_finish(&state->stream);
			  return;
			}
		      wrote = (off_t) (wrote + len);
		      state->data_len = 0;
		      break;
		    case 0:
		      break;
		    case -1:
		      eof = 1;
		      break;
		    }
		}
	    }
	  else
	    {
	      len = (long) (state->inputBuffer + state->buf_len - state->data_ptr);
	      if (len < 0)
		{
		  *error = MP3SPLT_ERROR_WHILE_READING_FILE;
		  mad_frame_finish(&state->frame);
		  mad_stream_finish(&state->stream);
		  return;
		}
	      if (fwrite(state->data_ptr, 1, len, file_output) < 0)
		{
		  *error = MP3SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
		  mad_frame_finish(&state->frame);
		  mad_stream_finish(&state->stream);
		  return;
		}
	      wrote = (off_t) (wrote + len);
	      state->data_len = 0;
	      begin = state->end;
	    }
	  
	  long split_begin_point = state->bytes;
	  //while not end of file, read write :
	  while (!eof)
	    {
	      off_t to_read = MP3SPLT_MAD_BSIZE;
	      if (end > 0)
		{
		  to_read = (end - state->bytes);
		  if (to_read <= 0)
		    {
		      status = 0;
		      break;
		    }
		  if (to_read > MP3SPLT_MAD_BSIZE)
		    to_read = MP3SPLT_MAD_BSIZE;
		}
	      
	      //we read the file input
	      if (feof(state->file_input) || 
		  ((state->data_len = 
		    fread(state->inputBuffer, 1, to_read, state->file_input))<=0))
		{
		  eof = 1;
		  status = 0;
		  break;
		}
	      
	      //we write to file output
	      if (fwrite(state->inputBuffer, 1, state->data_len, file_output)<0)
		{
		  status = -5;
		  break;
		}
	      
	      state->bytes+=state->data_len;
	      
	      update_progress(sp_state, (float)
			      (state->bytes-split_begin_point),
			      (float)(end-split_begin_point),
			      1,0,MP3SPLT_DEFAULT_PROGRESS_RATE);
	    }
	  
	  state->end = end;
	  
	  if (!eof)
	    {
	      switch (get_valid_frame(state))
		{
		case 1:
		  len = (long) (state->data_ptr - state->inputBuffer);
		  if (len < 0)
		    {
		      *error = MP3SPLT_ERROR_WHILE_READING_FILE;
		      mad_frame_finish(&state->frame);
		      mad_stream_finish(&state->stream);
		      return;
		    }
		  if (fwrite(state->inputBuffer, 1, len, file_output) <= 0)
		    {
		      *error = MP3SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
		      mad_frame_finish(&state->frame);
		      mad_stream_finish(&state->stream);
		      return;
		    }
		  break;
		case 0:
		  break;
		case -1:
		  eof = 1;
		  break;
		}
	    }
	}
      
      //we write id3 and other stuff
      if (file_output)
	{
	  if (id3)
	    {
	      if (fseeko(file_output, getid3v1(file_output), SEEK_END)
		  == -1)
		{
		  *error = MP3SPLT_ERROR_SEEKING_FILE;
		  return;
		}
	      if(fwrite(id3, 1, 128, file_output) <=0)
		{
		  *error = MP3SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
		  return;
		}
	    }
	  if (state->mp3file.xing > 0)
	    {
	      if (fseeko(file_output, state->mp3file.xing_offset+4, SEEK_SET)!=-1)
		{
		  unsigned long headw = (unsigned long) (state->frames - fbegin + 1); // Frames
		  fputc((headw >> 24) & 0xFF, file_output);
		  fputc((headw >> 16) & 0xFF, file_output);
		  fputc((headw >> 8) & 0xFF, file_output);
		  fputc((headw >> 0) & 0xFF, file_output);
		  headw = (unsigned long) (wrote); // Bytes
		  fputc((headw >> 24) & 0xFF, file_output);
		  fputc((headw >> 16) & 0xFF, file_output);
		  fputc((headw >> 8) & 0xFF, file_output);
		  fputc((headw >> 0) & 0xFF, file_output);
		}
	      else
		{
		  *error = MP3SPLT_ERROR_SEEKING_FILE;
		  return;
		}
	    }
	  fclose(file_output);
	}
    }
  //if seekable :
  else
    {
      off_t begin = 0, end = -1;
      //if framemode
      if (state->framemode)
	{
	  unsigned long fbegin, fend, adjust;
	  fbegin = fend = adjust = 0;
	  fbegin = fbegin_sec * state->mp3file.fps;
	  /*fprintf(stderr, "fps = %f\n",
		  state->mp3file.fps);
	  fprintf(stderr, "freq = %d\n",
		  state->mp3file.freq);
	  fprintf(stderr, "bitrate = %d\n",
	  state->mp3file.bitrate);*/
	  if (fend_sec != -1)
	    {
	      //if adjustoption
	      if (adjustoption)
		{
		  if (fend_sec != -1)
		    {
		      float adj = (float) (adjustoption);
		      float len = (fend_sec - fbegin_sec);
		      if (adj > len)
			{
			  adj = len;
			}
		      if (fend_sec > adj)
			{
			  fend_sec -= adj;
			}
		      adjust = (unsigned long) (adj * 100.f);
		    }
		  else 
		    {
		      adjust=0;
		    }
		}
	      fend = fend_sec * state->mp3file.fps;
	    }
	  else 
	    {
	      fend = 0xFFFFFFFF;
	    }
	  
	  if (state->end == 0) {
	    if (state->first)
	      {
		state->h.ptr = state->mp3file.firsthead.ptr;
		state->h.framesize = state->mp3file.firsthead.framesize;
		begin = state->mp3file.firsthead.ptr;
		state->first = 0;
	      }
	    
	    put_progress_text(sp_state,MP3SPLT_PROGRESS_PREPARE);
	    
	    /*fprintf(stderr,"fbegin = %lu\n",
	      fbegin);*/
	    // Finds begin by counting frames
	    while (state->frames < fbegin)
	      {
		begin = findhead(state, state->h.ptr + state->h.framesize);
		if (begin==-1)
		  {
		    *error = MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
		    mad_frame_finish(&state->frame);
		    mad_stream_finish(&state->stream);
		    return;
		  }
		if ((begin!=state->h.ptr + state->h.framesize)&&(state->syncerrors>=0)) 
		  state->syncerrors++;
		if ((state->syncdetect)&&(state->syncerrors>
					  MP3SPLT_MAXSYNC))
		  checksync(state);
		state->h = makehead (state->headw, state->mp3file, state->h, begin);
		state->frames++;
		
		//if we have adjust mode, then put only 25%
		//else put 50%
		if (adjustoption)
		  {
		    update_progress(sp_state,(float)(state->frames),
				    (float)fend, 8,
				    0,MP3SPLT_DEFAULT_PROGRESS_RATE);
		  }
		else
		  {
		    update_progress(sp_state,(float)(state->frames),
				    (float)fend,progress_adjust_val,
				    0,MP3SPLT_DEFAULT_PROGRESS_RATE);
		  }
	      }
	  }
	  else
	    {
	      begin = state->end;
	    }
	  
	  if (state->mp3file.len > 0)
	    if (begin >= state->mp3file.len) // If we can check, we just do that :)
	      {
		*error = MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
		mad_frame_finish(&state->frame);
		mad_stream_finish(&state->stream);
		return;
	      }
	  
	  put_progress_text(sp_state,MP3SPLT_PROGRESS_PREPARE);
	  
	  long int frames_begin = state->frames;
	  // Finds end by counting frames
	  while (state->frames <= fend)
	    {
	      state->frames++;
	      end = findhead(state, state->h.ptr + state->h.framesize);
	      if (end == -1) {
		end = state->h.ptr + state->h.framesize; // Last valid offset
		eof=1;
		break;
	      }
	    
	      if ((end!=state->h.ptr + state->h.framesize)&&(state->syncerrors>=0))
		{
		  state->syncerrors++;
		}
	      if ((state->syncdetect)&&(state->syncerrors>MP3SPLT_MAXSYNC))
		{
		  checksync(state);
		}
	      state->h = makehead (state->headw, state->mp3file, state->h, end);
	      
	      //if we have a progress callback function
	      if (sp_state->split.p_bar.progress != NULL)
		{
		  //time split only calculates the end of the 
		  //split
		  if (((sp_state->options.split_mode == 
			MP3SPLT_OPTION_TIME_MODE) || 
		       (sp_state->options.split_mode ==
			MP3SPLT_OPTION_SILENCE_MODE))
		      && (!sp_state->options.option_auto_adjust))
		    {
		      update_progress(sp_state,
				      (float)(state->frames-fbegin),
				      (float)(fend-fbegin),
				      progress_adjust_val,
				      0,MP3SPLT_DEFAULT_PROGRESS_RATE);
		    }
		  else
		    {
		      if (adjustoption)
			{
			  if(adjust)
			    {
			      if (sp_state->options.split_mode == 
				  MP3SPLT_OPTION_TIME_MODE)
				{
				  update_progress(sp_state,
						  (float)(state->frames-frames_begin),
						  (float)(fend-frames_begin),
						  4,0,MP3SPLT_DEFAULT_PROGRESS_RATE);
				}
			      else
				{
				  update_progress(sp_state,
						  (float)(state->frames-frames_begin),
						  (float)(fend-frames_begin),
						  8,1/(float)8,MP3SPLT_DEFAULT_PROGRESS_RATE);
				}
			    }
			}
		      else
			{
			  update_progress(sp_state,
					  (float)(state->frames-stopped_frames),
					  (float)(fend-stopped_frames),
					  progress_adjust_val,
					  0,MP3SPLT_DEFAULT_PROGRESS_RATE);
			}
		    }
		}
	      
	      //if adjust option, scans for silence
	      if ((adjust) && (state->frames >= fend))
		{
		  if (mp3_scan_silence(sp_state, end, 2 * adjust, threshold, 0.f, 0) > 0)
		    {
		      adjust = (unsigned long) (silence_position(state->silence_list, state->off) * state->mp3file.fps);
		    }
		  else
		    {
		      adjust = (unsigned long) (adjustoption * state->mp3file.fps);
		    }
		  fend += adjust;
		  end = findhead(state, end);
		  ssplit_free(&state->silence_list);
		  adjust=0;
		  //progress
		  put_progress_text(sp_state,MP3SPLT_PROGRESS_PREPARE);
		  stopped_frames = state->frames;
		}
	    }
	  
	  state->end = end;
	  
	  //if xing, we get xing
	  if (state->mp3file.xing > 0)
	    {
	      unsigned long headw;
	      headw = (unsigned long) (state->frames - fbegin + 1); // Frames
	      state->mp3file.xingbuffer[state->mp3file.xing_offset+4] = (headw >> 24) & 0xFF;
	      state->mp3file.xingbuffer[state->mp3file.xing_offset+5] = (headw >> 16) & 0xFF;
	      state->mp3file.xingbuffer[state->mp3file.xing_offset+6] = (headw >> 8) & 0xFF;
	      state->mp3file.xingbuffer[state->mp3file.xing_offset+7] = headw  & 0xFF;
	      if (end == -1)
		end = state->mp3file.len;
	      headw = (unsigned long) (end - begin + state->mp3file.xing); // Bytes
	      state->mp3file.xingbuffer[state->mp3file.xing_offset+8] = (headw >> 24) & 0xFF;
	      state->mp3file.xingbuffer[state->mp3file.xing_offset+9] = (headw >> 16) & 0xFF;
	      state->mp3file.xingbuffer[state->mp3file.xing_offset+10] = (headw >> 8) & 0xFF;
	      state->mp3file.xingbuffer[state->mp3file.xing_offset+11] = headw  & 0xFF;
	    }
	}
      else
	//if not framemode
	{
	  // If file is CBR we can trust bitrate to find position
	  //CBR = constant bit rate
	  if (state->end == 0) {
	    begin = (off_t) (fbegin_sec * state->mp3file.bitrate + state->mp3file.firsth);
	    // Finds first valid header. Mantain clean files.
	    begin = findvalidhead (state, begin);
	    if (begin==-1)
	      {
		*error = MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
		mad_frame_finish(&state->frame);
		mad_stream_finish(&state->stream);
		return;
	      }
	    if (tabsel_123[1 - state->mp3file.mpgid][state->mp3file.layer-1][c_bitrate(state->headw)] != 
		state->mp3file.firsthead.bitrate)
	      {
		check_bitrate = 1;
	      }
	  }
	  else 
	    {
	      begin = state->end;
	    }
	  
	  if (fend_sec != -1)
	    {
	      end = (off_t) (fend_sec * state->mp3file.bitrate + state->mp3file.firsth);
	      end = findvalidhead (state, end); // We take the complete frame
	      if (tabsel_123[1 - state->mp3file.mpgid][state->mp3file.layer-1][c_bitrate(state->headw)] != 
		  state->mp3file.firsthead.bitrate)
		check_bitrate = 1;
	    }
	  state->end = end;
	}
      
      status = split (sp_state, filename, begin, end, id3);
    }
  
  if (check_bitrate)
    {
      *error = MP3SPLT_MP3_MIGHT_BE_VBR;
      mad_frame_finish(&state->frame);
      mad_stream_finish(&state->stream);
      return;
    }
  
  if ((status == 0) && eof)
    {
      *error = MP3SPLT_OK_SPLITTED_MP3_EOF;
      mad_frame_finish(&state->frame);
      mad_stream_finish(&state->stream);
      return;
    }
  
  //returned error
  switch (status)
    {
    case -1:
      *error = MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
      break;
    case -5:
      *error = MP3SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
      break;
    case 0:
      *error = MP3SPLT_OK_SPLITTED_MP3;
      break;
    default:
      *error = status;
      break;
    }
  
  mad_frame_finish(&state->frame);
  mad_stream_finish(&state->stream);
}

//this function search for the id3v1 and id3v2
off_t adjustsync(mp3_state *state, off_t begin, off_t end)
{
  off_t position;
  position = begin;
  if (fseeko(state->file_input, position, SEEK_SET)==-1)
    {
      return (off_t) (-1);
    }
  
  // First we search for ID3v1
  while (position++ < end)
    {
      if (fgetc(state->file_input)=='T') {
	if (fgetc(state->file_input)=='A') {
	  if (fgetc(state->file_input)=='G')
	    return (position + 127);
	  else position++;
	}
	if (fseeko(state->file_input, -1, SEEK_CUR) == -1)
	  {
	    return (off_t) (-1);
	  }
      }
    }
  
  position = begin;
  if (fseeko(state->file_input, position, SEEK_SET)==-1)
    {
      return (off_t) (-1);
    }

  // Now we search for ID3v2
  while (position++ < end) 
    {
      if (fgetc(state->file_input)=='I')
	{
	  if (fgetc(state->file_input)=='D')
	    {
	      if (fgetc(state->file_input)=='3')
		{
		  return (position - 1);
		}
	      else 
		{
		  position++;
		}
	    }
	  if(fseeko(state->file_input, -1, SEEK_CUR)==-1)
	    {
	      return (off_t) (-1);
	    }
	}
    }
  
  return end;
}

//the function counts the number of sync error splits and puts them in
//state->syncerrors and puts the sync error splitpoints in the
//splitpoints 
int syncerror_split (splt_state *sp_state, off_t *splitpoints,
		     char *filename)
{
  off_t offset = 0;
  mp3_state *state = sp_state->mstate;
  
  put_progress_text(sp_state,MP3SPLT_PROGRESS_SEARCH_SYNC);
  
  state->h.ptr = state->mp3file.firsthead.ptr;
  state->h.framesize = state->mp3file.firsthead.framesize;
  splitpoints[0] = 0;
  
  //we get the file length for the progress
  struct stat file_statistics;
  if(stat(filename, &file_statistics) == 0)
    {
      //search for syncerrors and put in splitpoints
      while (state->syncerrors < MP3SPLT_MAXTRACKS)
	{
	  offset = findhead(state, state->h.ptr + state->h.framesize);
	  if (offset==-1)
	    break;
	  if (offset!=state->h.ptr + state->h.framesize)
	    {
	      state->syncerrors++;
	      splitpoints[state->syncerrors] = adjustsync(state, state->h.ptr, offset);
	      if (splitpoints[state->syncerrors] == -1)
		return MP3SPLT_MP3_ERR_SYNC;
	      offset = findvalidhead(state, splitpoints[state->syncerrors]);
	      if (getword(state->file_input, offset, SEEK_SET, &state->headw) == -1)
		return MP3SPLT_MP3_ERR_SYNC;
	    }
	  state->h = makehead (state->headw, state->mp3file, state->h, offset);
	  
	  if (sp_state->cancel_split)
	    return MP3SPLT_SPLIT_CANCELLED;
	  update_progress(sp_state,(float)(offset),
			  (float)(file_statistics.st_size),1,0,
			  MP3SPLT_DEFAULT_PROGRESS_RATE);
	}
      sp_state->mstate->end2 = file_statistics.st_size;
    }
  else
    {
      return MP3SPLT_MP3_ERR_SYNC;
    }
  
  if (state->syncerrors == 0)
    return MP3SPLT_MP3_ERR_NO_SYNC_FOUND;
  if (state->syncerrors == MP3SPLT_MAXTRACKS)
    return MP3SPLT_MP3_ERR_TOO_MANY_SYNC_ERR;
  
  splitpoints[++state->syncerrors] = -1;
  
  return MP3SPLT_MP3_SYNC_OK;
}

//used by mp3_scan_silence, and compare with threshold, returns 0 if
//silence spot > threshold, 1 otherwise
int mp3_silence(mp3_state *state, int channels, mad_fixed_t threshold)
{
  int i, j;
  mad_fixed_t sample;

  for (j=0; j<channels; j++) {
    for(i=0; i<state->synth.pcm.length; i++)
      {
	//get silence spot?
	sample = mad_f_abs(state->synth.pcm.samples[j][i]);
	state->temp_level = state->temp_level *0.999 + sample*0.001;
	if (sample > threshold)
	  return 0;
      }
  }
  return 1;
}

//scan for silence
int mp3_scan_silence (splt_state *sp_state, off_t begin, 
		      unsigned long length, float threshold, 
		      float min, short output)
{
  int len = 0, found = 0, shot;
  short first, flush = 0, stop = 0;
  unsigned long silence_begin = 0, silence_end = 0, time;
  off_t pos;
  mad_fixed_t th;
  
  mp3_state *state = sp_state->mstate;
  
  put_progress_text(sp_state,MP3SPLT_PROGRESS_SCAN_SILENCE);
  
  pos = begin;
  th = mad_f_tofixed(convertfromdB(threshold));
  
  if (fseeko(state->file_input, begin, SEEK_SET)==-1)
    return -1;
  
  first = output;
  shot = MP3SPLT_DEFAULTSHOT;
  
  //initialise mad stuff
  mad_stream_init(&state->stream);
  mad_frame_init(&state->frame);
  mad_synth_init(&state->synth);
  mad_timer_reset(&state->timer);
  
  state->temp_level = 0.0;
  state->avg_level = 0.0;
  state->n_stat = 0;
  
  //we do the effective scan
  do
    {
      switch (get_valid_frame(state))
	{
	case 1:
	  //1 we have a valid frame
	  //we get mad infos and put them in the state
	  mad_timer_add(&state->timer, state->frame.header.duration);
	  mad_synth_frame(&state->synth,&state->frame);
	  time = mad_timer_count(state->timer, MAD_UNITS_CENTISECONDS);
	  
	  if (length > 0)
	    {
	      if (time >= length)
		{
		  flush = 1;
		  stop = 1;
		}
	    }
	  
	  if ((!flush) && (mp3_silence(state, MAD_NCHANNELS(&state->frame.header), th)))
	    {
	      if (len == 0) silence_begin = time;
	      if (first == 0) len++;
	      if (shot < MP3SPLT_DEFAULTSHOT)
		shot+=2;
	      silence_end = time;
	    }
	  else {
	    if (len > MP3SPLT_DEFAULTSILLEN) {
	      if ((flush) || (shot <= 0))
		{
		  double begin_position, end_position;
		  begin_position = (double) (silence_begin / 100.f);
		  end_position = (double) (silence_end / 100.f);
		  if ((end_position - begin_position - min) >= 0.f)
		    {
		      ssplit_new(&state->silence_list, begin_position, end_position, len);
		      found++;
		    }
		  len = 0;
		  shot = MP3SPLT_DEFAULTSHOT;
		}
	    }
	    else len = 0;
	    if ((first) && (shot <= 0))
	      first = 0;
	    if (shot > 0) shot--;
	  }
	  
	  if (state->mp3file.len > 0)
	    {
	      pos = ftello(state->file_input);
	      
	      //if we don't have silence split,
	      //put the 1/4 of progress
	      if (sp_state->options.split_mode != 
		  MP3SPLT_OPTION_SILENCE_MODE)
		{
		  update_progress(sp_state,(float)(time),
				  (float)(length),
				  4,1/(float)4,
				  MP3SPLT_DEFAULT_PROGRESS_RATE);
		}
	      else
		{
		  //if we have cancelled the split
		  if (sp_state->cancel_split)
		    {
		      stop = 1;
		    }
		  update_progress(sp_state,(float)pos,
				  (float)(state->mp3file.len),
				  1,0,MP3SPLT_DEFAULT_PROGRESS_RATE);
		}
	    }
	  break;;
	case 0:
	  //0 we do nothing
	  break;
	case -1:
	  // -1 means eof
	  stop = 1;
	  break;
	}
    } while (!stop && (found < MP3SPLT_MAXTRACKS));
  //only if we have silence mode, we set progress to 100%
  if (sp_state->options.split_mode == 
      MP3SPLT_OPTION_SILENCE_MODE)
    {
      update_progress(sp_state,1.0,1.0,1,1,1);
    }
  
  //we finish with mad_
  mad_frame_finish(&state->frame);
  mad_stream_finish(&state->stream);
  mad_synth_finish(&state->synth);
  
  return found;
}

//frees the mp3_state structure,
//used in the splt_state_free() function
void mp3_state_free (mp3_state *state)
{
  if (state->mp3file.xingbuffer)
    free(state->mp3file.xingbuffer);
  ssplit_free(&state->silence_list);
  
  //we free the state
  free(state);
}
