/**********************************************************
 *
 * libmp3splt -- library based on mp3splt,
 *               for mp3/ogg splitting without decoding
 *
 * Copyright (c) 2002-2005 M. Trotta - <matteo.trotta@lib.unimib.it>
 * Copyright (c) 2005-2006 Munteanu Alexandru - io_alex_2002@yahoo.fr
 *
 * http://mp3splt.sourceforge.net
 *
 *********************************************************/

/**********************************************************
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307,
 * USA.
 *
 *********************************************************/

#include <sys/stat.h>
#include <string.h>
#include <math.h>

#include "mp3splt_utils.h"

//string s is freed, call with strdup for example
void libmp3splt_checkparse_outformat(splt_state *state,
				     char *str, int *error)
{
  int j = 0;
  //we clean out the output
  while (j <= MP3SPLT_OUTNUM)
    {
      memset(state->oformat.format[j],'\0',MP3SPLT_MAXOLEN);
      j++;
    }
  
  snprintf(state->oformat.format_char,255,
	   "%s", str);
  *error = parse_outformat(str,state);
  if (*error == MP3SPLT_OUTPUT_FORMAT_OK)
    {
      int i;
      i = (int) (log10((double) (state->split.splitnumber)));
      state->oformat.output_format_digits = (char) ((i+1) | 0x30);
    }
}

//cancels the current split
void libmp3splt_stop_split(splt_state *state)
{
  state->cancel_split = MP3SPLT_TRUE;
}

//create new state
//error is the possible error
splt_state *libmp3splt_create_new_state (int *error)
{
  splt_state *state = NULL;
  
  if (!state)
    {
      state = splt_state_new(state,error);
    }
  
  //if no error, continue
  if (*error >= 0)
    {
      splt_state_reset(state);
      //we put the default options for a normal split
      splt_state_put_default_options(state);
    }
  
  return state;
}

//returns the number of syncerrors found
//for the file filename and returns possible errors 
//as sync_result variable
int libmp3splt_get_mp3_syncerrors_number(splt_state *state,
					 char *filename,
					 int *sync_result)
{
  int errors_number = 0;
  
  //if the file is mp3 (not ogg)
  if(state->file_format == MP3SPLT_MP3_FORMAT)
    {
      FILE *file_input;
      //we open the file
      if ((file_input = fopen(filename, "rb")) != NULL)
	{
	  //we get the infos for the file
	  if((state = get_mp3_info(state, file_input,
				   sync_result))
	     != NULL)
	    {
	      off_t splitpoints[MP3SPLT_MAXTRACKS+1];
	      //we detect sync errors
	      *sync_result = 
		syncerror_split(state, splitpoints, filename);
	      errors_number = state->mstate->syncerrors;
	      //we put the syncerrors to 0
	      state->mstate->syncerrors = 0;
	      fclose(file_input);
	      //we free the mp3 state after the check
	      mp3_state_free (state->mstate);
	    }
	  else
	    {
	      fclose(file_input);
	    }
	}
      else
	{
	  *sync_result = MP3SPLT_ERROR_CANNOT_OPEN_FILE;
	}
    }
  else
    {
#ifndef NO_OGG
      *sync_result = MP3SPLT_ERROR_CANNOT_SYNC_OGG;
#endif
    }
  
  return errors_number;
}

//get freedb search
//returns the results
//search is not freed
freedb_results *libmp3splt_get_freedb_search(splt_state *state,
					     //our search
					     char *search,
					     //possible errors
					     int *error)
{
  //we suppose we dont search more than ~1000 chars
  char search_plus_spaces[1024] = "";
  char temp[3];
  
  //transform " " to "+"
  int i;
  for (i = 0; i < strlen(search); i++)
    {
      if (search[i] == ' ')
	{
	  strcat(search_plus_spaces, "+");
	}
      else
	{
	  snprintf(temp,2,"%c", search[i]);
	  strcat(search_plus_spaces, temp);
	}
    }
  
  //puts the results in "search_results"
  *error = process_freedb_search(state,
				 search_plus_spaces);
  
  return state->fdb.search_results;
}

//must be called after get_freedb_search,
//otherwise, it will fail (seg fault!?)
//result must be freed
//returns the content of a cddb file
//you need to write it on the disk in 
//a cddb file to use it
//we return possible errors in err
//the cddb_file is the file to write
void libmp3splt_write_freedb_file_result(splt_state *state, int disc_id,
					 char *cddb_file, int *err)
{
  char *freedb_file_content;
  freedb_file_content =
    get_freedb_file(state, disc_id, err);
  
  //we write the result to the file
  FILE *output;
  if (!(output = fopen(cddb_file, "w")))
    {
      *err = MP3SPLT_CANNOT_WRITE_CDDB_FILE;
    }
  else
    {
      fprintf (output,"%s",freedb_file_content);
      fclose(output);
    }
}

//we get the cue splitpoints from the file
//returns the max splitpoints
//returns possible error in err
int libmp3splt_get_cue_splitpoints_from_file(splt_state *state,
					     char *file,
					     int *err)
{
  //max splits
  int max_splits = 0;
  
  //we empty the artist and the album
  //state->tags.id.album[0] = '\0';
  //state->tags.id.artist[0] = '\0';
  
  //we get the splitpoints
  max_splits = get_cue_splitpoints(file, state, err);
  
  return max_splits;
}

//we get the cddb splitpoints from the file
//returns the max splitpoints
int libmp3splt_get_cddb_splitpoints_from_file(splt_state *state,
					      char *file,
					      int *err)
{
  //max splits
  int max_splits = 0;
  
  //we get the splitpoints
  max_splits = get_cddb_splitpoints(file, state, err);
  
  return max_splits;
}

//returned result must be freed
//number of the wrapped files in wrap_files_number
//and possible error in process_result
//free the results with
//free_files(files,numbers);
char **libmp3splt_get_wrap_files(splt_state *state,
				 char *filename,
				 int *process_result,
				 int *wrap_files_number)
{
  FILE *file_input;
  //we try to open the file read binary
  if((file_input = fopen(filename, "rb")) != NULL)
    {
      //TRUE means we only get the files
      //first NULL because no directory
      dewrap(file_input,MP3SPLT_TRUE, NULL, process_result,
	     NULL);
      //close the file
      fclose(file_input);
      *wrap_files_number = state->wrap.wrap_files_num;
    }
  else
    {
      *process_result = MP3SPLT_ERROR_CANNOT_OPEN_FILE;
    }
  
  return state->wrap.wrap_files;
}

//returns if the library has id3tag support or not
int libmp3splt_has_libid3tag()
{
#ifdef NO_ID3TAG
  return MP3SPLT_FALSE;
#else
  return MP3SPLT_TRUE;
#endif
}

//returns if the library has ogg support or not
int libmp3splt_has_ogg()
{
#ifdef NO_OGG
  return MP3SPLT_FALSE;
#else
  return MP3SPLT_TRUE;
#endif
}

//main function, split the file
int libmp3splt_split_file(splt_state *state, char *filename, 
			  int splitnumber, char *the_filename_path)
{
  state->cancel_split = MP3SPLT_FALSE;
  
  //the result of the split
  int result = MP3SPLT_OK;
  //the new filename path
  char *new_filename_path = NULL;
  
  //we put splitnumber in the state, we will use it later
  state->split.splitnumber = splitnumber;
  state->split.current_split = 0;
  
  //we check if the filename is a real file
  if (!is_file(filename))
    {
      result = MP3SPLT_ERROR_INEXISTENT_FILE;
      return result;
    }
  
  //if the new_filename_path is "", we put the directory of
  //the current song
  new_filename_path =
    check_put_dir_of_cur_song(filename,
			      the_filename_path);
  
  //we use strdup in the check_put_dir..
  if (new_filename_path == NULL)
    {
      result = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
      return result;
    }
  else
    {
      //checks and sets correct options
      check_set_correct_options(state);
      
      //if we have compatible options
      //this function is optional,
      if (!check_compatible_options(state))
	{
	  result = MP3SPLT_ERROR_INCOMPATIBLE_OPTIONS;
	  return result;
	}
      
      check_if_mp3_or_ogg(state, filename,
			  &result);
      if (result < 0)
	{
	  return result;
	}
      
      //check means the test is ok
      check_if_new_filename_path_correct(new_filename_path,
					 &result);
#ifdef __WIN32__
      //manage c:\ because the gtk dir returns us "c:\"
      //and the normal directories without the "\"
      if (new_filename_path[strlen(new_filename_path)-1] == '\\')
	{
	  new_filename_path[strlen(new_filename_path)-1] = '\0';
	}
#endif
      
      song_put_total_time(filename,
			  state, &result);
      
      if (result < 0)
	{
	  return result;
	}
      
      //the type of the split
      switch (state->options.split_mode)
	{
	case MP3SPLT_OPTION_WRAP_MODE:
	  wrap_split(filename,
		     new_filename_path,
		     &result, state);
	  break;
	case MP3SPLT_OPTION_SILENCE_MODE:
	  silence_split(filename,
			new_filename_path,
			state, &result);
	  break; 
	case MP3SPLT_OPTION_TIME_MODE:
	  time_split(filename,
		     new_filename_path,
		     state, &result);
	  break;
	default:
	  //this is the normal split or error mode split:
	  //here we also have mp3_error_mode and
	  //mp3_frame_mode 
	  //if we dont have error mode, check if we have
	  //at least 2 splitpoints
	  if (state->options.split_mode != 
	      MP3SPLT_OPTION_MP3_ERROR_MODE)
	    {
	      //check if we have at least 2 splitpoints
	      if (state->split.splitnumber < 2)
		{
		  result = MP3SPLT_ERROR_SPLITPOINTS;
		  return result;
		}
	    }
      
	  check_if_splitpoints_in_order(state, &result);
	  //we check if the splitpoints are in order
	  if (result < 0)
	    {
	      return result;
	    }
      
	  //total time of the song
	  check_splitpts_inf_song_length(state, 
					 &result);
	  
	  //if no error, continue
	  if (result < 0)
	    {
	      return result;
	    }
	  
	  //do the effective multiple split
	  multiple_split(filename, state, 
			 new_filename_path,
			 &result);
	  break;
	}
      
      //free memory
      free(new_filename_path);
    }
  
  return result;
}

//puts a splitpoint in the state with an eventual file name
//if name == NULL, we don't put the filename
//number is which splitpoint ?
void libmp3splt_put_splitpoint(splt_state *state, int number,
			       long splitpoint, char *name)
{
  state->split.splitpoints[number] = splitpoint;
  if (name == NULL)
    {
      snprintf(state->split.fn[number],255,"%s","");
    }
  else
    {
      snprintf(state->split.fn[number],255,name);
    }
}

//this function frees the left variables in the library
//call this function ONLY at the end of the program
//and don't forget to call it
void libmp3splt_free_left_variables(splt_state *state)
{
  //free the wrapped files
  free_files(state->wrap.wrap_files,
	     state->wrap.wrap_files_num);
  //we free the search results
  free_search_results(state->fdb.cdstate,
		      state->fdb.search_results);
  //free our state
  splt_state_free(state);
}

void libmp3splt_v311()
{
  
}
