/**********************************************************
 *
 * libmp3splt -- library based on mp3splt,
 *               for mp3/ogg splitting without decoding
 *
 * Copyright (c) 2002-2005 M. Trotta - <matteo.trotta@lib.unimib.it>
 * Copyright (c) 2005-2006 Munteanu Alexandru - io_alex_2002@yahoo.fr
 *
 * http://mp3splt.sourceforge.net
 *
 *********************************************************/

/**********************************************************
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307,
 * USA.
 *
 *********************************************************/

#include <sys/types.h>
#include <mad.h>
#include <stdlib.h>
#include <stdio.h>

#ifndef NO_OGG
#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>
#endif

//maximum number of tracks that we can split
#define MP3SPLT_MAXTRACKS 1000
#define MP3SPLT_MAXSYNC MP3SPLT_MAXTRACKS

/******************************/
/* Structures for the freedb  */

//structure that defines one result from freedb
typedef struct {
  //name of the result
  char *name;
  //revisions of the result
  int *revisions;
  //how many revisions
  int revision_number;
  //unique identifier
  int id;
} one_result;

//structure used for the freedb results
typedef struct {
  one_result *results;
  //how many results
  int number;
} freedb_results;

//maximum number of freedb search
#define MP3SPLT_MAXCD 512
//maximum length of the disc id
#define MP3SPLT_DISCIDLEN 8
//structure for the freedb search
struct cd {
  char discid[MP3SPLT_DISCIDLEN+1];
  char category[20];
};

typedef struct {
  struct cd discs[MP3SPLT_MAXCD];
  int foundcd;
} cd_state;

//structure containing everything used for the
//freedb search
typedef struct {
  //we stock here the results of the freedb search
  freedb_results *search_results;
  //we stock the state of the CD
  //(for the freedb search)
  cd_state *cdstate;
} splt_freedb;

/******************************/
/* Structures for the wrap    */

//structure related to the wrapping
typedef struct {
  //for the wrap
  char **wrap_files;
  int wrap_files_num;
} splt_wrap;

/***************************************/
/* Structures for the output format    */

#define MP3SPLT_MAXOLEN 255
#define MP3SPLT_OUTNUM  10

//structure defining the output format
typedef struct {
  //format as @n_@t.. as a string
  char format_char[1024];
  //when we have @n option on output format
  char output_format_digits;
  //format for the cddb cue output
  char format[MP3SPLT_OUTNUM+1][MP3SPLT_MAXOLEN];
} splt_oformat;

/***************************/
/* Structures for the tags */

//structure for the mp3 id3v1 tags
struct id3v1 {
  //it's 31 for id3tag
  char title[128];
  //it's 31 for id3tag
  char artist[128];
  //it's 31 for id3tag
  char album[128];
  char year[5];
  char comment[31];
  short track;
  unsigned char genre;
};

//structure related to tags
typedef struct {
  //id3 and vorbis tags before writing them to file
  char *song_tags;
  struct id3v1 id[MP3SPLT_MAXTRACKS+1];
  char performer[MP3SPLT_MAXTRACKS+1][128];
} splt_tags;

/**********************************/
/* Structure for the split        */

#define MP3SPLT_DEFAULT_PROGRESS_RATE 350
#define MP3SPLT_DEFAULT_PROGRESS_RATE2 50

typedef struct {
  //maximum number of characters for the filename(without the
  //extension) when displaying the progress
  //warning, don't set this more than 512 !
  //default is 40
  int progress_text_max_char;
  //filename that we are currently splitting
  char filename_shorted[512];
  //the current percent of the progress
  float percent_progress;
  //the progress type
  //can be : 
  //MP3SPLT_PROGRESS_PREPARE
  //MP3SPLT_PROGRESS_CREATE
  //MP3SPLT_PROGRESS_SEARCH_SYNC
  //MP3SPLT_PROGRESS_SCAN_SILENCE
  int progress_type;
  //float = fraction of how much %
  //char * = name on the progress bar
  void (*progress)(int);
} mp3splt_progress;

typedef struct {
  //total time of the song
  long total_time;
  //the part of file that we are currently splitting
  //1 if the first, 2 if the second
  //the number of splitpoints
  int current_split;
  //how many splits, this should not be manually set
  int splitnumber;
  //put this function if you want that the library
  //tells you when a file has been splitted
  //the char* is the filename
  void (*file_splitted)(char *);
  //for the progress bar
  mp3splt_progress p_bar;
  //sends a message to the main program to tell him what
  //he is doing
  //possible values : 
  //MP3SPLT_FRAME_MODE_ENABLED
  void (*put_message)(int);
  //the splitpoints
  long splitpoints[2*MP3SPLT_MAXTRACKS+1];
  //the name of the files to split
  char fn[MP3SPLT_MAXTRACKS+1][256];
} splt_struct;

/**********************************/
/* Ogg structures                 */

#ifndef NO_OGG
typedef struct {
  int length;
  unsigned char *packet;
} v_packet;

typedef struct {
  ogg_sync_state *sync_in;
  ogg_stream_state *stream_in;
  vorbis_dsp_state *vd;
  vorbis_info *vi;
  vorbis_block *vb;
  int prevW;
  ogg_int64_t initialgranpos;
  ogg_int64_t len;
  ogg_int64_t cutpoint_begin;
  struct ssplit *silence_list;
  unsigned int serial;
  v_packet **packets; /* 2 */
  v_packet **headers; /* 3 */
  OggVorbis_File vf;
  vorbis_comment vc;
  FILE *in,*out;
  short end;
  float off;
  double avg_level;
  float temp_level;
  unsigned long n_stat;
} ogg_state;
#endif

// Struct that will contain header's useful infos
struct header {
  off_t ptr;	// Offset of header
  int bitrate;
  int padding;
  int framesize;
};

/**********************************/
/* Mp3 structures                 */

#define MP3SPLT_MAD_BSIZE 4032

// Struct that will contains infos on mp3 and an header struct of first valid header
struct mp3 {
  int mpgid;	// 0 or 1
  int layer;	// mpg1, mpg2, or mpg3
  int channels;
  //0 = single channel
  //1 = dual channel
  //2 = joint stereo
  //3 = stereo
  //4 = other
  //frequency
  int freq;
  //bitrate
  int bitrate;
  //frames per second
  float fps;
  //used for the xing header
  int xing;
  char *xingbuffer;
  off_t xing_offset;
  //length of the mp3 file
  off_t len;
  //where we begin reading
  off_t firsth;
  struct header firsthead;
};

typedef struct {
  FILE *file_input;
  struct header h;
  //if we are in framemode or not
  short framemode;
  //total frames
  unsigned long frames;
  int syncdetect;
  //number of sync errors found
  //(syncerror mode)
  unsigned long syncerrors;
  off_t end;
  off_t end2;
  off_t bytes;
  int first;
  unsigned long headw;
  
  //see the mp3 structure
  struct mp3 mp3file;
  //see the ssplit structure
  struct ssplit *silence_list;
  
  //used internally, libmad structures
  struct mad_stream stream;
  struct mad_frame frame;
  struct mad_synth synth;
  //internally used by the silence detection functions
  mad_fixed_t temp_level;
  //the offset
  float off;
  //used internally when reading the file
  unsigned char inputBuffer[MP3SPLT_MAD_BSIZE];
  //mad timer
  mad_timer_t timer;
  //used internally, pointer to the beginning of a frame
  unsigned char *data_ptr;
  //used internally, length of a frame
  long data_len;
  //length of a buffer when reading a frame
  int buf_len;
  double avg_level;
  unsigned long n_stat;
} mp3_state;

/**********************************/
/* Options structure              */

//normal split
#define MP3SPLT_OPTION_NORMAL_MODE 1
//option to split the file created with mp3wrap
#define MP3SPLT_OPTION_WRAP_MODE 2
//silence mode, to split with silence detection.
#define MP3SPLT_OPTION_SILENCE_MODE 3
//error mode (mp3 only). It is useful to split large file derivated
//from a concatenation of smaller files. When you have a file to
//split, you should always try to use this option.
#define MP3SPLT_OPTION_MP3_ERROR_MODE 4
//this option will create an indefinite number of smaller files with
//a fixed time length specified by split_time in seconds
#define MP3SPLT_OPTION_TIME_MODE 5

//default parameters
#define MP3SPLT_DEFAULT_PARAM_THRESHOLD -48.0
#define MP3SPLT_DEFAULT_PARAM_OFFSET 0.8
#define MP3SPLT_DEFAULT_PARAM_MINIMUM_LENGTH 0.0
#define MP3SPLT_DEFAULT_PARAM_GAP 30
#define MP3SPLT_DEFAULT_PARAM_TRACKS 0

//tags options
#define MP3SPLT_TAGS_ORIGINAL_FILE 0
//tags issued from the cddb or cue for example
#define MP3SPLT_CURRENT_TAGS 1
#define MP3SPLT_NO_TAGS 2

#define MP3SPLT_DEFAULT_OUTPUT "@a - @n - @t"

//structure with all the options supplied to split the file
typedef struct {
  //this can take the following values :
  //MP3SPLT_OPTION_NORMAL_MODE
  //MP3SPLT_OPTION_WRAP_MODE
  //MP3SPLT_OPTION_SILENCE_MODE
  //MP3SPLT_OPTION_MP3_ERROR_MODE
  //MP3SPLT_OPTION_TIME_MODE
  int split_mode;
  
  //might be :
  //MP3SPLT_TAGS_ORIGINAL_FILE - write tags from original file
  //MP3SPLT_NO_TAGS - does not write any tags
  //MP3SPLT_CURRENT_TAGS - tags issued from the cddb or cue for example
  int tags;
  
  //option_mins_secs is to write the splitted filenames with 
  //mins_secs_hundr instead of the filenames supplied in splt_state->fn
  int option_mins_secs;
  
  //may be TRUE or FALSE. If TRUE, when using cddb or cue, we set the
  //filename as the default : @a - @n - @t
  //where @a is the performer @p if we found one on the cddb file
  int option_output_is_default;
  
  //frame mode (mp3 only). Process all frames, seeking split positions
  //by counting frames and not with bitrate guessing.
  int option_mp3_frame_mode;
  //the time of split when split_mode = OPTION_TIME_SPLIT
  float split_time;
  //this option uses silence detection to auto-adjust splitpoints.
  int option_auto_adjust;
  //input not seekable. disabling This allows you to split mp3 and ogg streams
  //which can be read only one time and can’t be seeked.
  //WARNING!
  //if you don't know what this means, set it to FALSE
  int option_input_not_seekable;
  
  //PARAMETERS---------------------------------------
  //PARAMETERS for option_auto_adjust and option_silence_mode :
  //the sound level to be considered silence
  //(it is a float number between -96 and 0. Default is -48 dB)
  float parameter_threshold;
  //the offset of cutpoint in silence
  //Float number between -2 and 2 and allows you to adjust the offset
  //of cutpoint in silence time.0 is the begin of silence, and 1 the
  //end;default is 0.8. 
  float parameter_offset;
  
  //PARAMETERS for option_silence_mode :
  //the desired number of tracks
  //(positive integer number of tracks to be splitted;by default all
  //tracks are splitted)
  int parameter_number_tracks;
  //the minimum silence length in seconds
  //(positive float of the minimum number of seconds to be considered
  //a valid splitpoint)
  float parameter_minimum_length;
  //allows you to remove the silence between splitted tracks
  int parameter_remove_silence;
  
  //PARAMETERS for option_auto_adjust :
  //the gap value around splitpoint to search for silence
  //(positive integer for the time to decode before and after
  //splitpoint;default gap is 30 seconds)
  int parameter_gap;
} splt_options;

#define MP3SPLT_MP3_FORMAT 1
#define MP3SPLT_OGG_FORMAT 2

/**********************************/
/* Main structure                 */

//structure for the splt state
typedef struct {
  //if we have mp3, ogg or other to split
  //values are :
  //MP3SPLT_MP3_FORMAT
  //MP3SPLT_OGG_FORMAT
  int file_format;
  //if we cancel split or not
  int cancel_split;
  
  //split related
  splt_struct split;
  //options for the split
  splt_options options;
  //song tags related
  splt_tags tags;
  //tags of the original file to split
  struct id3v1 original_tags;
  //wrap related
  splt_wrap wrap;
  //freedb related
  splt_freedb fdb;
  //output format  
  splt_oformat oformat;
  
  //file format states, mp3,ogg..
  mp3_state *mstate;
#ifndef NO_OGG
  ogg_state *ostate;
#endif
} splt_state;

/*****************************************/
/* Confirmations, errors and messages    */

//error and confirmation messages :
//sync
#define MP3SPLT_MP3_MIGHT_BE_VBR 301
#define MP3SPLT_MP3_SYNC_OK 300

#define MP3SPLT_MP3_ERR_SYNC -300
#define MP3SPLT_MP3_ERR_NO_SYNC_FOUND -301
#define MP3SPLT_MP3_ERR_TOO_MANY_SYNC_ERR -302

//freedb
#define MP3SPLT_FREEDB_MAX_CD_REACHED 104
#define MP3SPLT_FREEDB_CUE_OK 103
#define MP3SPLT_FREEDB_CDDB_OK 102
#define MP3SPLT_FREEDB_FILE_OK 101
#define MP3SPLT_FREEDB_OK 100

#define MP3SPLT_FREEDB_ERROR_INITIALISE_SOCKET -101
#define MP3SPLT_FREEDB_ERROR_CANNOT_GET_HOST -102
#define MP3SPLT_FREEDB_ERROR_CANNOT_OPEN_SOCKET -103
#define MP3SPLT_FREEDB_ERROR_CANNOT_CONNECT -104
#define MP3SPLT_FREEDB_ERROR_CANNOT_SEND_MESSAGE -105
#define MP3SPLT_FREEDB_ERROR_INVALID_SERVER_ANSWER -106
#define MP3SPLT_FREEDB_ERROR_SITE_201 -107
#define MP3SPLT_FREEDB_ERROR_SITE_200 -108
#define MP3SPLT_FREEDB_ERROR_BAD_COMMUNICATION -109
#define MP3SPLT_FREEDB_ERROR_GETTING_INFOS -110
#define MP3SPLT_FREEDB_NO_CD_FOUND -111
#define MP3SPLT_FREEDB_ERROR_CANNOT_RECV_MESSAGE -112
#define MP3SPLT_CUE_ERROR_CANNOT_OPEN_FILE_READING -113
#define MP3SPLT_CDDB_ERROR_CANNOT_OPEN_FILE_READING -114
#define MP3SPLT_INVALID_CUE_FILE -115
#define MP3SPLT_INVALID_CDDB_FILE -116
#define MP3SPLT_CANNOT_WRITE_CDDB_FILE -117

//wrap
#define MP3SPLT_DEWRAP_OK 200

#define MP3SPLT_DEWRAP_ERR_FILE_LENGTH -200
#define MP3SPLT_DEWRAP_ERR_VERSION_OLD -201
#define MP3SPLT_DEWRAP_ERR_NO_FILE_OR_BAD_INDEX -202
#define MP3SPLT_DEWRAP_ERR_FILE_DAMAGED_INCOMPLETE -203
#define MP3SPLT_DEWRAP_ERR_FILE_NOT_WRAPED_DAMAGED -204
#define MP3SPLT_DEWRAP_ERR_CANNOT_CREATE_DIRECTORY -204

//main
#define MP3SPLT_OK_SPLITTED_OGG_EOF 9
#define MP3SPLT_OK_SPLITTED_MP3_EOF 8
#define MP3SPLT_NO_SILENCE_SPLITPOINTS_FOUND 7
#define MP3SPLT_TIME_SPLIT_OK 6
#define MP3SPLT_SILENCE_OK 5
#define MP3SPLT_SPLITPOINT_BIGGER_THAN_LENGTH 4
#define MP3SPLT_OK_SPLITTED_OGG 2
#define MP3SPLT_OK_SPLITTED_MP3 1
#define MP3SPLT_OK 0

#define MP3SPLT_ERROR_SPLITPOINTS -1
#define MP3SPLT_ERROR_CANNOT_OPEN_FILE -2
#define MP3SPLT_ERROR_INVALID_MP3 -3
#define MP3SPLT_ERROR_INVALID_OGG -4
#define MP3SPLT_ERROR_EQUAL_SPLITPOINTS -5
#define MP3SPLT_ERROR_SPLITPOINTS_NOT_IN_ORDER -6
#define MP3SPLT_ERROR_NEGATIVE_SPLITPOINT -7
#define MP3SPLT_ERROR_INCORRECT_PATH -8
#define MP3SPLT_ERROR_CANNOT_SYNC_OGG -9
#define MP3SPLT_ERROR_INCOMPATIBLE_OPTIONS -10
#define MP3SPLT_ERROR_SILENCE -11
#define MP3SPLT_ERROR_INPUT_OUTPUT_SAME_FILE -12
#define MP3SPLT_ERROR_TIME -13
#define MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY -15
#define MP3SPLT_ERROR_CANNOT_OPEN_DEST_FILE -16
#define MP3SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE -17
#define MP3SPLT_ERROR_WHILE_READING_FILE -18
#define MP3SPLT_ERROR_SEEKING_FILE -19
#define MP3SPLT_ERROR_BEGIN_OUT_OF_FILE -20
#define MP3SPLT_ERROR_INEXISTENT_FILE -21
#define MP3SPLT_SPLIT_CANCELLED -22
#define MP3SPLT_ERROR_OGG_WRAP_NOT_IMPLEMENTED -23

//output format
#define MP3SPLT_OUTPUT_FORMAT_OK 400

#define MP3SPLT_OUTPUT_FORMAT_ERROR -400
#define MP3SPLT_OUTPUT_FORMAT_AMBIGUOUS -401

//miscellaneous messages
#define MP3SPLT_FRAME_MODE_ENABLED 0

//progress messages
#define MP3SPLT_PROGRESS_PREPARE 0
#define MP3SPLT_PROGRESS_CREATE 1
#define MP3SPLT_PROGRESS_SEARCH_SYNC 2
#define MP3SPLT_PROGRESS_SCAN_SILENCE 3

/* other useful variables */
//backslash character
#ifndef MP3SPLT_DIRCHAR
#ifdef _WIN32
#define MP3SPLT_DIRCHAR '\\'
#define MP3SPLT_NDIRCHAR '/'
#else
#define MP3SPLT_DIRCHAR '/'
#define MP3SPLT_NDIRCHAR '\\'
#endif
#endif

#define MP3SPLT_TRUE 1
#define MP3SPLT_FALSE 0


/* other stuff :/ */
#define MP3SPLT_READBSIZE 1024
#define MP3SPLT_DEFAULTSILLEN 10
#define MP3SPLT_DEFAULTSHOT 25

#define MP3SPLT_VARCHAR '@'

#define MP3SPLT_MP3EXT ".mp3"
#define MP3SPLT_MP3EXTU ".MP3"
#define MP3SPLT_OGGEXTU ".OGG"
#define MP3SPLT_OGGEXT ".ogg"

#define MP3SPLT_BUFSIZE 4096
