/**********************************************************
 *
 * libmp3splt -- library based on mp3splt,
 *               for mp3/ogg splitting without decoding
 *
 * Copyright (c) 2002-2005 M. Trotta - <matteo.trotta@lib.unimib.it>
 * Copyright (c) 2005-2006 Munteanu Alexandru - io_alex_2002@yahoo.fr
 *
 * http://mp3splt.sourceforge.net
 *
 *********************************************************/

/**********************************************************
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307,
 * USA.
 *
 *********************************************************/

#include <sys/stat.h>
#include <string.h>
#include <math.h>

#include "mp3splt_utils.h"

//get the mp3 info and put it in the state
//error is the possible error
splt_state *get_mp3_info(splt_state *state, FILE *file_input,
			 int *error)
{
  //checks if valid mp3 file
  //last argument, if framemode or not
  state->mstate = mp3info(file_input, state, 
			  state->options.option_mp3_frame_mode,
			  error);
  //if error
  if (*error < 0)
    {
      return NULL;
    }
  return state;
}

#ifndef NO_OGG
//get ogg info and put it in the state
//error is the possible error
splt_state *get_ogg_info(splt_state *state, FILE *file_input,
			 int *error)
{
  //checks if valid ogg file
  state->ostate = ogginfo(file_input, state->ostate,
			  error);
  //if error
  if (*error < 0)
    {
      return NULL;
    }
  
  return state;
}
#endif

//put the total time in state->split.total_time
void song_put_total_time(char *filename, 
			 splt_state *state,
			 int *error)
{
  FILE *file_input;
  
  //if we can open the file
  if ((file_input = fopen(filename, "rb")) != NULL)
    {
#ifndef NO_OGG
      //if the file is ogg
      if (state->file_format == MP3SPLT_OGG_FORMAT)
	{
	  OggVorbis_File ogg_file;
	  
	  //if we can open file
	  if(ov_open(file_input, &ogg_file, NULL, 0) >= 0)
	    {
	      state->split.total_time = (long)
		ov_time_total(&ogg_file, -1) * 100;
	      ov_clear(&ogg_file);
	    }
	}
      else
#endif
	//if the file is mp3
	{
	  //if we can read the file
	  //get the mp3 info
	  if((state = get_mp3_info(state, file_input,error)) 
	     != NULL)
	    {
	      //we free the mp3 state after the check
	      mp3_state_free (state->mstate);
	    }
	  fclose(file_input);
	}
    }
  else
    {
      *error = MP3SPLT_ERROR_CANNOT_OPEN_FILE;
    }
}

//the real split of the file
void real_simple_split(char *filename, splt_state *state,
		       double splt_beg, double splt_end,
		       int *error, char *output_fname)
{
  FILE *file_input;
  int split_error = *error;
  
  //if we can open the file
  if ((file_input = fopen(filename, "rb")) != NULL)
    {
#ifndef NO_OGG
      //if the file is ogg
      if (state->file_format == MP3SPLT_OGG_FORMAT)
	{
	  if((state = get_ogg_info(state, file_input,error))
	     != NULL)
	    {
	      state->ostate->off = state->options.parameter_offset;
	      
	      put_ogg_tags(state,&split_error);
	      
	      if (error >= 0)
		{
		  //effective ogg split
		  oggsplit(output_fname, state,
			   splt_beg, splt_end,
			   !state->options.option_input_not_seekable,
			   state->options.parameter_gap,
			   state->options.parameter_threshold,
			   &split_error);
		}
	      
	      ogg_state_free(state->ostate);
	    }
	  else
	    {
	      //necessary --^ ?
	      fclose(file_input);
	    }
	}
      else
#endif
	//if the file is mp3
	{
	  if((state = 
	      get_mp3_info(state, file_input,
			   error)) != NULL)
	    {	  
	      state->mstate->off = 
		state->options.parameter_offset;	 
	      
	      //we initialise frames to 1
	      if (state->split.total_time > 0)
		{
		  state->mstate->frames = 1;
		}
	      
	      //put id3v1 tags
	      char *id3_data;
	      char id3buffer[128];
	      id3_data = put_mp3_song_tags(id3buffer, filename,
					   state, &split_error);
	      if (split_error >= 0)
		{
		  //effective mp3 split
		  mp3split(output_fname, state, id3_data,
			   splt_beg, splt_end, &split_error);
		}
	      
	      //we free the mp3 state after the check
	      mp3_state_free (state->mstate);
	    }
	  fclose(file_input);
	}
      
      //automatically set progress callback to 100%
      update_progress(state,1.0,1.0,1,1,1);
      
      //we treat the split error
      if (split_error < 0)
	{
	  *error = split_error;
	}
      else
	{
	  //we put the splitted file
	  put_splitted_file(state, output_fname);
	  
	  if (*error != MP3SPLT_SPLITPOINT_BIGGER_THAN_LENGTH)
	    {
	      *error = split_error;
	    }
	}
    }
  else
    {
      *error = MP3SPLT_ERROR_CANNOT_OPEN_FILE;
    }
}

//simple split with only 2 splitpoints
//i is the current splitpoint
//(used to get the filename)
//error in error parameter
void simple_split(char *filename, splt_state *state,
		  long split_begin, long split_end,
		  char *new_filename_path,
		  int i, int *error)
{
  char *complete_filename = NULL;
  //begin splitpoint
  double splt_beg;
  //end splitpoint
  double splt_end;
  
  if (split_begin < state->split.total_time)
    {
      if (split_end > state->split.total_time)
	{
	  split_end = state->split.total_time;
	}
      
      complete_filename =
	get_complete_new_filename(filename, state,
				  split_begin, split_end,
				  new_filename_path,
				  i,error);
      
      //if no error
      if (*error >= 0)
	{
	  //if input and output are not the same file
	  if (strcmp(filename,complete_filename) != 0)
	    {
	      //if the first splitpoint different than the end point
	      if (split_begin != split_end)
		{
		  //convert to float for hundredth
		  // 34.6  --> 34 seconds and 6 hundredth
		  splt_beg = split_begin / 100;
		  splt_beg += ((split_begin % 100) / 100.);
		  splt_end = split_end / 100;
		  splt_end += ((split_end % 100) / 100.);
		  
		  //we do the real split, if mp3 or ogg..
		  real_simple_split(filename,state,
				    splt_beg,splt_end,error,
				    complete_filename);
		}
	    }
	  //if input and output are same file
	  else
	    {
	      *error = MP3SPLT_ERROR_INPUT_OUTPUT_SAME_FILE;
	    }
	}
      
      //freeing memory
      free(complete_filename);
    }
  else
    {
      *error = MP3SPLT_ERROR_BEGIN_OUT_OF_FILE;
    }
}

//splits the file with multiple points
void multiple_split(char *filename, splt_state *state, 
		    char *new_filename_path, int *error)
{
  int i = 0;
  char *final_fname = NULL;
  int split_result;
  
  //if we have the sync error mode
  //we do a different split than the normal split
  if (state->options.split_mode == MP3SPLT_OPTION_MP3_ERROR_MODE)
    {
#ifndef NO_OGG
      if (state->file_format == MP3SPLT_OGG_FORMAT)
	{
	  *error = MP3SPLT_ERROR_CANNOT_SYNC_OGG;
	}
      else
#endif
	{
	  FILE *file_input;
	  //we try to open the file
	  if ((file_input = fopen(filename, "rb")) != NULL)
	    {
	      //we get the infos for the file
	      if((state = get_mp3_info(state, file_input, error))
		 != NULL)
		{
		  off_t splitpoints[MP3SPLT_MAXTRACKS+1];
		  //we detect sync errors
		  *error = syncerror_split(state, splitpoints,
					   filename);
	      
		  //automatically set progress callback to 100%
		  update_progress(state,1.0,1.0,1,1,1);
		  
		  //if no error
		  if (error >= 0)
		    {
		      int malloc_number = 
			strlen(new_filename_path) + 30;
		      
		      //allocate memory
		      if((final_fname = malloc(malloc_number))
			 != NULL)
			{
			  //we put the number of sync errors
			  state->split.splitnumber = state->mstate->syncerrors+1;
			  //we split all sync errors
			  for (i = 0; i < state->mstate->syncerrors; i++)
			    {
			      //if we don't cancel the split
			      if (!state->cancel_split)
				{
				  //we put the current file to split
				  state->split.current_split = i+1;
			      
				  //default filenames for now
				  //TODO output format ..
				  snprintf(final_fname, malloc_number,
					   "%s%cTrack %02d"MP3SPLT_MP3EXT,
					   new_filename_path,MP3SPLT_DIRCHAR,i+1);
			      
				  //we put the files in the splitted files
				  //we put the filename to split
				  snprintf(state->split.fn[i], 255,"%s",
					   get_real_name(final_fname));
				  //cut extension
				  state->split.fn[i][strlen(state->split.fn[i])-4] = '\0';
				  
				  //we split with the detected splitpoints
				  split_result =
				    split(state,final_fname, splitpoints[i],
					  splitpoints[i+1], NULL);
				  
				  //automatically set progress callback to 100%
				  update_progress(state,1.0,1.0,1,1,1);
				  
				  //if the split has been a success
				  if (split_result >= 0)
				    {
				      put_splitted_file(state,
							final_fname);
				    }
			      
				  //we put the real error
				  switch (split_result)
				    {
				    case MP3SPLT_OK_SPLITTED_MP3:
				      *error = MP3SPLT_MP3_SYNC_OK;
				      break;
				    case MP3SPLT_ERROR_BEGIN_OUT_OF_FILE:
				      *error = MP3SPLT_MP3_SYNC_OK;
				      break;
				    default:
				      *error = MP3SPLT_MP3_ERR_SYNC;
				      break;
				    }
				}
			      //if we cancel the split
			      else
				{
				  *error = MP3SPLT_SPLIT_CANCELLED;
				}
			    }
			}
		      else
			{
			  *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
			}
		      //free memory
		      free(final_fname);
		    }
		  //we put the syncerrors to 0
		  state->mstate->syncerrors = 0;
		  fclose(file_input);
		  //we free the mp3 state after the split
		  mp3_state_free (state->mstate);
		}
	      else
		{
		  fclose(file_input);
		}
	    }
	  else
	    {
	      *error = MP3SPLT_ERROR_CANNOT_OPEN_FILE;
	    }
	}
    }
  else
    //if we don't have the sync error mode
    //we do a normal split
    {
      int old_option_mins = state->options.option_mins_secs;
      //for every 2 splitpoints, split
      for(i = 0; i < (state->split.splitnumber - 1); i++)
	{
	  //if we don't cancel the split
	  if (!state->cancel_split)
	    {
	      //if we have no filename, we put
	      //option mins secs
	      if (state->split.fn[i][0] == '\0')
		{
		  state->options.option_mins_secs = MP3SPLT_TRUE;
		}
	      else
		{
		  state->options.option_mins_secs = old_option_mins;
		}
	      
	      simple_split(filename, state,
			   state->split.splitpoints[i],
			   state->split.splitpoints[i+1],
			   new_filename_path,
			   i, error);
	    }
	  else
	    {
	      *error = MP3SPLT_SPLIT_CANCELLED;
	    }
	}
    }
}

//do the wrap split
void wrap_split(char *filename,
		char *new_filename_path,
		int *error, splt_state *state)
{
  //only for mp3
  if(state->file_format == MP3SPLT_MP3_FORMAT)
    {
      //we free possibly previous wrapped files
      free_files(state->wrap.wrap_files,
		 state->wrap.wrap_files_num);
      state->wrap.wrap_files_num = 0;
      
      FILE *file_input;
      //we check if we can open the file in check_if_mp3_or_ogg
      //open the file read binary
      if ((file_input = fopen(filename, "rb")) != NULL)
	{
	  //MP3SPLT_FALSE means don't list tracks
	  //we get the result error or confirmation
	  dewrap(file_input,MP3SPLT_FALSE, new_filename_path, 
		 error, state);
	  //close the file
	  fclose(file_input);
	}
      else
	{
	  *error = MP3SPLT_ERROR_CANNOT_OPEN_FILE;
	}
    }
  else
    {
      *error = MP3SPLT_ERROR_OGG_WRAP_NOT_IMPLEMENTED;
    }
}

//returns the number of silence splits found
//or the number of tracks specified in the options
//sets the silence splitpoints in state->split.splitpoints
//write_tracks is a boolean, MP3SPLT_TRUE if we call the function
//write_silence_tracks and MP3SPLT_FALSE otherwise
int set_silence_splitpoints(char *filename, splt_state *state,
			    int write_tracks, int *error)
{
  //found is the number of silence splits found
  int found = 0;
  //checks if the state is null
  int state_null = MP3SPLT_FALSE;
  struct ssplit *temp = NULL, **list = NULL;
  FILE *file_input;
  //we try to open the file
  if((file_input = fopen(filename, "rb")))
    {
      //we get information and scan for silence
      if(state->file_format == MP3SPLT_MP3_FORMAT)
	{
	  if((state = get_mp3_info(state, file_input,
				   error))
	     != NULL)
	    {
	      state->mstate->off = state->options.parameter_offset;
	      
	      list = &state->mstate->silence_list;
	      found = mp3_scan_silence(state, state->mstate->mp3file.firsthead.ptr, 0,
				       state->options.parameter_threshold,
				       state->options.parameter_minimum_length, 1);	      
	      //TODO mp3_scan_silence error
	    }
	  else
	    {
	      state_null = MP3SPLT_TRUE;
	      *error = MP3SPLT_ERROR_INVALID_MP3;
	    }
	}
#ifndef NO_OGG
      else
	{
	  if((state = get_ogg_info(state, file_input,error))
	     != NULL)
	    {
	      state->ostate->off = state->options.parameter_offset;
	      
	      list = &state->ostate->silence_list;
	      found = ogg_scan_silence(state, 0,
				       state->options.parameter_threshold,
				       state->options.parameter_minimum_length,
				       1, NULL, 0);
	      //TODO ogg_scan_silence error
	    }
	  else
	    {
	      state_null = MP3SPLT_TRUE;
	      *error = MP3SPLT_ERROR_INVALID_OGG;
	    }
	}
#endif
      //if we have a state not NULL
      if(!state_null)
	{
	  //we set the number of tracks
	  int order = 0;
	  if (!state->cancel_split)
	    {
	      found++;
	      if ((state->options.parameter_number_tracks > 0)&&
		  (state->options.parameter_number_tracks < MP3SPLT_MAXTRACKS))
		{
		  if (state->options.parameter_number_tracks < found)
		    {
		      found = state->options.parameter_number_tracks;
		    }
		}
	      state->split.splitpoints[0] = 0;
	      
	      temp = *list;int i;
	      //we take all splitpoints found and we remove silence 
	      //if needed
	      for (i = 1; i < found; i++)
		{
		  if (temp == NULL)
		    {
		      found = i;
		      break;
		    }
		  if (state->options.parameter_remove_silence)
		    {
		      state->split.splitpoints[2*i-1] = 
			(long)(temp->begin_position * 100);
		      state->split.splitpoints[2*i] = 
			(long)(temp->end_position * 100);
		    }
		  else
		    {
		      state->split.splitpoints[i] = 
			(long) (silence_position(temp,
						 state->options.parameter_offset) *100);
		    }
		  temp = temp->next;
		}
	  
	      //we order the splitpoints
	      if (state->options.parameter_remove_silence)
		{
		  order = (found-1)*2+1;
		}
	      else 
		{
		  order = found;
		}
	      
	      order_splitpoints(state->split.splitpoints, order);
	      
	      //last splitpoint, end of file
	      state->split.splitpoints[order] = state->split.total_time;
	    }
	  else
	    {
	      *error = MP3SPLT_SPLIT_CANCELLED;
	    }
	  
	  //free memory if we don't call the write_silence_tracks function
	  if ((!write_tracks) ||
	      (*error == MP3SPLT_SPLIT_CANCELLED))
	    {
	      //if mp3 we close the file that we opened in the function
	      //set_silence_splitpoints
	      if (state->file_format == MP3SPLT_MP3_FORMAT)
		{
		  fclose(state->mstate->file_input);
		  mp3_state_free (state->mstate);
		}
#ifndef NO_OGG
	      else
		{
		  ogg_state_free(state->ostate);
		}
#endif
	    }
	}
      else
	{
	  fclose(file_input);
	}
    }
  
  return found;
}

//write the silence tracks
//warning, this function must always be called after
//set_silence_splitpoints
//we have possible error in error
void write_silence_tracks(int found, char *filename,
			  char *new_filename_path,
			  splt_state *state, int *error)
{
  char *final_fname = NULL;
  int i,malloc_number = 
    strlen(new_filename_path) + 30;
  
  //begin splitpoint and end splitpoint
  double beg_pos = 0, end_pos = 0;
  
  if((final_fname = malloc(malloc_number))
     != NULL)
    {
      //put frames to 1 if necessary
      if(state->file_format == MP3SPLT_MP3_FORMAT)
	{
	  if (state->split.total_time > 0)
	    {
	      state->mstate->frames = 1;
	    }
	}
      
      //we put the number of tracks found
      state->split.splitnumber = found+1;
      
      //we write all found tracks
      for (i = 0; i < found; i++)
	{
	  if (!state->cancel_split)
	    {
	      //we put the correct filename
	      if (state->file_format == MP3SPLT_MP3_FORMAT)
		{
		  snprintf(final_fname,malloc_number,
			   "%s%cTrack %02d"MP3SPLT_MP3EXT,
			   new_filename_path,MP3SPLT_DIRCHAR,i+1);
		}
	      else
		{
		  snprintf(final_fname,malloc_number,
			   "%s%cTrack %02d"MP3SPLT_OGGEXT,
			   new_filename_path,MP3SPLT_DIRCHAR,i+1);
		}
	      
	      //we put the filename to split
	      snprintf(state->split.fn[i], 255,"%s",
		       get_real_name(final_fname));
	      //cut extension
	      state->split.fn[i][strlen(state->split.fn[i])-4] = '\0';
	      
	      long split_begin,split_end;
	      if (state->options.parameter_remove_silence)
		{
		  split_begin = state->split.splitpoints[2*i];
		  split_end = state->split.splitpoints[2*i+1];
		  
		  if (state->file_format == MP3SPLT_MP3_FORMAT) 
		    {
		      state->mstate->end = 0;
		    }
#ifndef NO_OGG
		  else 
		    {
		      state->ostate->end = 0;
		    }
#endif
		}
	      else
		{
		  split_begin = state->split.splitpoints[i];
		  split_end = state->split.splitpoints[i+1];
		}
	      
	      beg_pos = get_double_pos(split_begin);
	      end_pos = get_double_pos(split_end);
	      
	      //effective silence split
	      if (state->file_format == MP3SPLT_MP3_FORMAT)
		{
		  //put id3
		  char *id3_data;
		  char id3buffer[128];
		  id3_data = put_mp3_song_tags(id3buffer,
					       filename,
					       state,error);
		  
		  if (*error >= 0)
		    {
		      state->options.parameter_gap = 0;
		      state->options.parameter_threshold = 0;
		      mp3split (final_fname, state,
				id3_data, beg_pos, end_pos,
				error);
		    }
		}
#ifndef NO_OGG
	      else 
		{
		  put_ogg_tags(state,error);
		  
		  oggsplit(final_fname,
			   state, beg_pos, end_pos, 
			   !state->options.option_input_not_seekable, 
			   0, 0, error);
		}
#endif
	      //put the splitted file if no error
	      if (*error >= 0)
		{
		  put_splitted_file(state,
				    final_fname);
		}
	      
	      //we put the silence split errors
	      switch (*error)
		{
		case MP3SPLT_MP3_MIGHT_BE_VBR:
		  *error = MP3SPLT_SILENCE_OK;
		  break;
		case MP3SPLT_OK_SPLITTED_MP3:
		  *error = MP3SPLT_SILENCE_OK;
		  break;
		case MP3SPLT_OK_SPLITTED_OGG:
		  *error = MP3SPLT_SILENCE_OK;
		  break;
		case MP3SPLT_OK_SPLITTED_MP3_EOF:
		  *error = MP3SPLT_SILENCE_OK;
		  break;
		case MP3SPLT_OK_SPLITTED_OGG_EOF:
		  *error = MP3SPLT_SILENCE_OK;
		  break;
		default:
		  *error = MP3SPLT_ERROR_SILENCE;
		  break;
		}
	    }
	  else
	    {
	      *error = MP3SPLT_SPLIT_CANCELLED;
	    }
	}
      
      //free memory
      free(final_fname);
      
      //if mp3 we close the file that we opened in the function
      //set_silence_splitpoints
      if (state->file_format == MP3SPLT_MP3_FORMAT)
	{
	  fclose(state->mstate->file_input);
	  mp3_state_free (state->mstate);
	}
#ifndef NO_OGG
      else
	{
	  ogg_state_free(state->ostate);
	}
#endif
    }
  else
    {
      *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
    }
}

//do the silence split
//possible error in error
void silence_split(char *filename, char *new_filename_path,
		   splt_state *state, int *error)
{
  int found = 0;
  found = set_silence_splitpoints(filename,state,
				  MP3SPLT_TRUE, error);
  
  //if no error
  if(*error >= 0)
    {
      //if we have found splitpoints, write the silence tracks
      if (found > 1)
	{
	  write_silence_tracks(found,filename,new_filename_path,
			       state,error);
	}
      else
	{
	  //if mp3 we close the file that we opened in the function
	  //set_silence_splitpoints
	  if (state->file_format == MP3SPLT_MP3_FORMAT)
	    {
	      fclose(state->mstate->file_input);
	      mp3_state_free (state->mstate);
	    }
#ifndef NO_OGG
	  else
	    {
	      ogg_state_free(state->ostate);
	    }
#endif
	  *error = MP3SPLT_NO_SILENCE_SPLITPOINTS_FOUND;
	}
    }
}

//function used with the -t option (time split
//create an indefinite number of smaller files with a fixed time
//length specified by options.split_time in seconds
void time_split(char *filename, char *new_filename_path,
		splt_state *state, int *error)
{
  char *final_fname = NULL;
  //if the state song has an error
  int state_null = MP3SPLT_FALSE;
  FILE *file_input;
  int j=0,tracks=1;
  //the begin split
  double begin = 0.f;
  double end = (double)state->options.split_time;
  int malloc_number = 
    strlen(new_filename_path) + 30;
  
  //we check if we can open the file in check_if_mp3_or_ogg
  //we open the file
  if ((file_input = fopen(filename, "rb")) != NULL)
    {
      //if the file is mp3 (not ogg)
      if(state->file_format == MP3SPLT_MP3_FORMAT)
	{
	  //we get the infos for the file
	  if((state = get_mp3_info(state, file_input, error))
	     == NULL)
	    state_null = MP3SPLT_TRUE;
	  
	  //for the progress bar
	  struct stat file_statistics;
	  if(stat(filename, &file_statistics) == 0)
	    {
	      state->mstate->end2 = file_statistics.st_size;
	    }
	  else
	    {
	      *error = MP3SPLT_ERROR_CANNOT_OPEN_FILE;
	    }
	}
#ifndef NO_OGG
      else
	//if ogg
	{
	  //we get the infos for the file
	  if((state = get_ogg_info(state, file_input,error))
	     == NULL)
	    {
	      state_null = MP3SPLT_TRUE;
	    }
	}
#endif
  
      //if no state error
      if (!state_null)
	{
	  if((final_fname = malloc(malloc_number))
	     != NULL)
	    {
	      //initialise frames to 1 after the get_mp3_info
	      if (state->file_format == MP3SPLT_MP3_FORMAT)
		{
		  if (state->split.total_time > 0)
		    {
		      state->mstate->frames = 1;
		    }
		}
	  
	      //TODO, split_time in hundr of secs
	      state->split.splitnumber = (int)
		floor(((state->split.total_time/(float)100) /
		       (state->options.split_time))+1) + 1;
	      
	      //while we have tracks
	      do {
		//if we don't cancel the split
		if (!state->cancel_split)
		  {
		    //we put the correct filename
		    if(state->file_format == MP3SPLT_MP3_FORMAT)
		      {
			snprintf(final_fname,malloc_number,
				 "%s%cTrack %02d"MP3SPLT_MP3EXT,
				 new_filename_path,MP3SPLT_DIRCHAR,j+1);
		      }
		    else
		      {
			snprintf(final_fname,malloc_number,
				 "%s%cTrack %02d"MP3SPLT_OGGEXT,
				 new_filename_path,MP3SPLT_DIRCHAR,j+1);
		      }
		
		    //we put the filename to split
		    snprintf(state->split.fn[tracks-1], 255,"%s",
			     get_real_name(final_fname));
		    //cut extension
		    state->split.fn[tracks-1][strlen(state->split.fn[tracks-1])-4] = '\0';
		    
		    if (state->file_format == MP3SPLT_MP3_FORMAT) 
		      {
			if (*error >= 0)
			  {
			    //put id3
			    char *id3_data;
			    char id3buffer[128];
			    id3_data = put_mp3_song_tags(id3buffer,
							 filename,
							 state, error);
			    if (*error >= 0)
			      {
				//we put the end of the file - we need this for the progress bar
				mp3split (final_fname, state,
					  id3_data, begin, end,
					  error);
			      }
			  }
		      }
#ifndef NO_OGG
		    else
		      {
			put_ogg_tags(state,error);
		    
			oggsplit(final_fname, 
				 state, begin, end, 
				 !state->options.option_input_not_seekable,
				 state->options.parameter_gap,
				 state->options.parameter_threshold,
				 error);
		      }
#endif
		    //if no error for the split, put the splitted file
		    if (*error >= 0)
		      {
			put_splitted_file(state, 
					  final_fname);
		      }
		
		    //set new splitpoints
		    begin=end;
		    end+= state->options.split_time;
		    tracks++;
	    
		    //get out
		    if ((*error==MP3SPLT_ERROR_BEGIN_OUT_OF_FILE)||
			(*error==MP3SPLT_MP3_MIGHT_BE_VBR)||
			(*error==MP3SPLT_OK_SPLITTED_MP3_EOF)||
			(*error==MP3SPLT_OK_SPLITTED_OGG_EOF))
		      {
			tracks = 0;
		      }
		    if (*error==MP3SPLT_ERROR_BEGIN_OUT_OF_FILE)
		      {
			j--;
		      }
		  }
		else
		  {
		    *error = MP3SPLT_SPLIT_CANCELLED;
		  }
	      }  while (j++<tracks);
	  
	      //we put the mp3 time split error
	      switch (*error)
		{
		case MP3SPLT_MP3_MIGHT_BE_VBR : 
		  *error = MP3SPLT_TIME_SPLIT_OK;
		  break;
		case MP3SPLT_OK_SPLITTED_OGG : 
		  *error = MP3SPLT_TIME_SPLIT_OK;
		  break;
		case MP3SPLT_OK_SPLITTED_MP3 : 
		  *error = MP3SPLT_TIME_SPLIT_OK;
		  break;
		case MP3SPLT_OK_SPLITTED_OGG_EOF : 
		  *error = MP3SPLT_TIME_SPLIT_OK;
		  break;
		case MP3SPLT_OK_SPLITTED_MP3_EOF : 
		  *error = MP3SPLT_TIME_SPLIT_OK;
		  break;
		case MP3SPLT_ERROR_BEGIN_OUT_OF_FILE : 
		  *error = MP3SPLT_TIME_SPLIT_OK;
		  break;
		case MP3SPLT_SPLIT_CANCELLED :
		  *error = MP3SPLT_SPLIT_CANCELLED;
		  break;
		default:
		  *error = MP3SPLT_ERROR_TIME;
		  break;
		}
	  
	      //free memory
	      free(final_fname);
	    }
	  else
	    {
	      *error = MP3SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
	    }
      
	  //we free the mp3 state after the check
	  if(state->file_format == MP3SPLT_MP3_FORMAT)
	    {
	      fclose(file_input);
	      mp3_state_free (state->mstate);
	    }
#ifndef NO_OGG
	  else
	    {
	      ogg_state_free(state->ostate);
	    }
#endif
	}
      else
	{
	  fclose(file_input);
	}
    }
  else
    {
      *error = MP3SPLT_ERROR_CANNOT_OPEN_FILE;
    }
}



